import { __decorate } from 'tslib';
import { EventEmitter, ViewChildren, ViewChild, Input, Output, HostListener, Component, ViewEncapsulation, Injectable, Pipe, Directive, forwardRef, ElementRef, IterableDiffers, NgModule, LOCALE_ID } from '@angular/core';
import { DatePipe, registerLocaleData, CommonModule } from '@angular/common';
import { NG_VALUE_ACCESSOR, FormBuilder, FormsModule, ReactiveFormsModule } from '@angular/forms';
import localeTr from '@angular/common/locales/tr';
import { TranslateHttpLoader } from '@ngx-translate/http-loader';
import { TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { HttpClient } from '@angular/common/http';
import { MatNativeDateModule, MAT_DATE_LOCALE } from '@angular/material/core';
import { BrowserModule } from '@angular/platform-browser';
import { MatTooltipModule } from '@angular/material/tooltip';
import { ClPagerModule } from 'cl-angular-pager';
import { Dictionary, UUID, Utils, SearchCriteria, BaseRepository, ResponseModel, ClCoreModule } from 'cl-angular-core';
import { ComboboxSettings, ComboboxStyles, PromiseParameters, ClComboboxModule } from 'cl-angular-combobox';
import { ClCheckboxModule } from 'cl-angular-checkbox';
import { ClDatePickerModule } from 'cl-angular-date-picker';
import * as alasql from 'alasql';
import { utils } from 'alasql';
import * as XLSX from 'xlsx';
import { MatDialog } from '@angular/material/dialog';
import { Router, ActivatedRoute } from '@angular/router';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

class DataTableStorageModel {
    constructor() {
        this.isFullScreen = false;
        this.isScrollable = true;
        this.isCompressed = true;
        this.pageIndex = 0;
        this.pageCount = 50;
    }
    static parse(element) {
        let item = new DataTableStorageModel();
        if (element != undefined) {
            item.filters = element[DataTableStorageModel.filtersKey];
            item.columns = element[DataTableStorageModel.columnsKey];
            item.isFullScreen = element[DataTableStorageModel.isFullScreenKey];
            item.isScrollable = element[DataTableStorageModel.isScrollableKey];
            item.isCompressed = element[DataTableStorageModel.isCompressedKey];
            item.pageIndex = element[DataTableStorageModel.pageIndexKey];
            item.pageCount = element[DataTableStorageModel.pageCountKey];
            if (item.pageCount == 0) {
                item.pageCount = 50;
            }
        }
        return item;
    }
}
DataTableStorageModel.filtersKey = "filters";
DataTableStorageModel.columnsKey = "columns";
DataTableStorageModel.isFullScreenKey = "isFullScreen";
DataTableStorageModel.isScrollableKey = "isScrollable";
DataTableStorageModel.isCompressedKey = "isCompressed";
DataTableStorageModel.pageIndexKey = "pageIndex";
DataTableStorageModel.pageCountKey = "pageCount";

class LocalStorageHandler {
    static getUserRowguid() {
        return LocalStorageHandler.getStringForKey(LocalStorageHandler.userRowguidKey);
    }
    static getUserId() {
        var id = LocalStorageHandler.getStringForKey(LocalStorageHandler.userIdKey);
        return id ? Number(id) : 0;
    }
    static setValueForKey(key, value) {
        if (value === undefined) {
            localStorage.removeItem(key);
        }
        else {
            localStorage.setItem(key, value);
        }
    }
    static getStringForKey(key, defaultValue = null) {
        if (localStorage === undefined) {
            return defaultValue;
        }
        const value = localStorage.getItem(key);
        if (value === undefined || value === null) {
            return defaultValue;
        }
        return value;
    }
    static setOriginalUserID(userID) {
        if (userID == undefined) {
            return undefined;
        }
        else {
            localStorage.setItem(LocalStorageHandler.originalUserIDKey, userID.toString());
        }
    }
    static getOriginalUserID() {
        return localStorage.getItem(LocalStorageHandler.originalUserIDKey);
    }
    static setDataTableSettings(dataTable) {
        if (dataTable.filters != undefined)
            dataTable.filters = dataTable.filters.filter(x => x.value != undefined);
        localStorage.setItem(LocalStorageHandler.dataTableSettingsKey + "_" + dataTable.dataTableID, JSON.stringify(dataTable));
    }
    static getDataTableSettings(dataTableID) {
        let dataTableSettings = localStorage.getItem(LocalStorageHandler.dataTableSettingsKey + "_" + dataTableID);
        dataTableSettings = JSON.parse(dataTableSettings);
        let dataTableStorageModel = DataTableStorageModel.parse(dataTableSettings);
        dataTableStorageModel.dataTableID = dataTableID;
        return dataTableStorageModel;
    }
    static removeDataTableFilter(dataTableID) {
        let dtSetting = this.getDataTableSettings(dataTableID);
        dtSetting.filters = undefined;
        this.setDataTableSettings(dtSetting);
        return dtSetting;
    }
}
LocalStorageHandler.userNameKey = 'userName';
LocalStorageHandler.userEmailKey = "userEmail";
LocalStorageHandler.tokenKey = 'token';
LocalStorageHandler.sessionKey = 'session';
LocalStorageHandler.userFullNameKey = 'userFullName';
LocalStorageHandler.userIdKey = 'userId';
LocalStorageHandler.userRowguidKey = 'userRowguid';
LocalStorageHandler.userTypeKey = 'userType';
LocalStorageHandler.userPhotoPathKey = 'userPhotoPath';
LocalStorageHandler.originalUserIDKey = 'originalUserID';
LocalStorageHandler.departments = "departments";
LocalStorageHandler.dataTableSettingsKey = 'dataTableSettings';

var SelectionState;
(function (SelectionState) {
    SelectionState[SelectionState["Selected"] = 0] = "Selected";
    SelectionState[SelectionState["PartiallySelected"] = 1] = "PartiallySelected";
    SelectionState[SelectionState["NotSelected"] = 2] = "NotSelected";
})(SelectionState || (SelectionState = {}));
class FilterItem {
    constructor() {
        this.selected = false;
        this.enabled = true;
        this.activated = false;
        this.selectionState = SelectionState.NotSelected;
        this.readonly = false;
        this.observers = new Array();
        this.childSelectionText = '';
        this.maxChildDisplayLength = 3;
        this.childSelectionTexts = [];
    }
    toggleSelection() {
        if (this.children !== undefined && this.children.length === 0) {
            this.setSelected(!this.selected);
        }
    }
    setSelected(value) {
        this.selected = value;
        this.setSelectionState();
        this.observers.forEach(observer => observer.selectedChanged(this));
    }
    setSelectionState() {
        if (this.selected) {
            this.selectionState = SelectionState.Selected;
        }
        else {
            this.selectionState = SelectionState.NotSelected;
        }
    }
    getSelected() {
        return this.selected;
    }
    static parseList(objects) {
        if (objects == undefined)
            return undefined;
        let items = new Array();
        objects.forEach(element => {
            items.push(FilterItem.parse(element));
        });
        return items;
    }
    static parse(element) {
        if (element == undefined)
            return undefined;
        var filterItem = new FilterItem();
        filterItem.key = element['key'];
        filterItem.value = element['value'];
        return filterItem;
    }
    selectedChanged(item) {
        var selectedChildren = this.children.filter(child => child.selectionState === SelectionState.Selected);
        if (selectedChildren.length === 0) {
            const hasPartiallySelectedChild = this.children.some(child => child.selectionState === SelectionState.PartiallySelected);
            if (hasPartiallySelectedChild) {
                this.selectionState = SelectionState.PartiallySelected;
            }
            else {
                this.selectionState = SelectionState.NotSelected;
            }
        }
        else if (this.children.length === selectedChildren.length) {
            this.selectionState = SelectionState.Selected;
        }
        else {
            this.selectionState = SelectionState.PartiallySelected;
        }
        item.setParentSelectionText();
        this.observers.forEach(observer => observer.selectedChanged(this));
    }
    setParentSelectionText() {
        var text = this.key;
        var shouldAdd = this.selectionState === SelectionState.Selected;
        if (this.children === undefined || this.children.length > 0) {
            return;
        }
        var activeItem = this.parent;
        while (activeItem !== undefined) {
            var index = activeItem.childSelectionTexts.indexOf(text);
            if (index > -1 && !shouldAdd) {
                activeItem.childSelectionTexts.splice(index, 1);
            }
            else if (index === -1 && shouldAdd) {
                activeItem.childSelectionTexts.push(text);
            }
            if (activeItem.childSelectionTexts.length >= this.maxChildDisplayLength) {
                activeItem.childSelectionText = activeItem.childSelectionTexts.length + 'adet seçili';
            }
            else {
                activeItem.childSelectionText = activeItem.childSelectionTexts.join(', ');
            }
            activeItem = activeItem.parent;
        }
    }
    setAndObserveChildren(children) {
        this.children = new Array();
        this.addAndObserveChildren(children);
    }
    addAndObserveChildren(children) {
        children.forEach(child => this.addAndObserveChild(child));
    }
    refreshSelectionState() {
        if (this.children !== undefined && this.children.length > 0) {
            this.children.forEach(child => {
                child.refreshSelectionState();
                this.selectedChanged(child);
            });
        }
    }
    addAndObserveChild(child) {
        if (this.children === undefined) {
            this.children = new Array();
        }
        this.children.push(child);
        if (child.observers.indexOf(this) === -1) {
            child.observers.push(this);
        }
    }
    getSelectedChildren() {
        const selectedItems = [];
        if (this.children !== undefined && this.children.length === 0) {
            if (this.selectionState === 0) {
                if (selectedItems.indexOf(this) === -1) {
                    selectedItems.push(this);
                }
            }
            return selectedItems;
        }
        else if (this.children !== undefined && this.children.length > 0) {
            this.children.forEach(item => {
                selectedItems.push(...item.getSelectedChildren());
            });
        }
        return selectedItems;
    }
}
Array.prototype.asFilter = function () {
    return this.map(x => x.asFilterItem(x));
};
Array.prototype.clone = function () {
    return this.map(x => x.clone());
};

class Filter {
    constructor() {
        this.loading = true;
        // 0 : text
        // 1 : numeric
        // 2 : date
        // 3 : bool
        // 4 : select
        this.type = 0;
        this.allowMultipleSelections = true;
        this.sourceTableSearchKeys = ["Name"];
        this.autoFilterRequestEnabled = true;
    }
    static parseList(objects) {
        if (objects == undefined)
            return undefined;
        let items = new Array();
        objects.forEach(element => {
            items.push(Filter.parse(element));
        });
        return items;
    }
    static parse(element) {
        if (element == undefined)
            return undefined;
        var filter = new Filter();
        filter.type = element["type"];
        filter.sourceTable = element["sourceTable"];
        if (element["sourceTableSearchKeys"] != undefined)
            filter.sourceTableSearchKeys = element["sourceTableSearchKeys"];
        filter.placeholder = element["displayText"];
        filter.filterKey = element["filterKey"];
        if (element["autoFilterRequestEnabled"] != undefined)
            filter.autoFilterRequestEnabled = element["autoFilterRequestEnabled"];
        filter.items = FilterItem.parseList(element["items"]);
        if (filter.items !== undefined && filter.items.length > 0)
            filter.loading = false;
        var valueMap = element["valueMap"];
        if (valueMap !== undefined) {
            filter.valueMap = new Dictionary();
            valueMap.forEach(map => {
                if (map["key"] !== undefined && map["value"] !== undefined)
                    filter.valueMap.add(map["key"], map["value"]);
            });
        }
        return filter;
    }
}

class Condition {
    constructor() {
        this.shouldOperationFallbackToDefaultTemplate = true;
        this.direction = true;
    }
    static parseList(objects) {
        if (objects == undefined)
            return undefined;
        let items = new Array();
        objects.forEach(element => {
            items.push(Condition.parse(element));
        });
        return items;
    }
    static parse(element) {
        if (element == undefined)
            return undefined;
        var condition = new Condition();
        condition.condition = element["condition"];
        condition.operationCondition = element["operationCondition"];
        condition.field = element["field"];
        condition.template = element["template"];
        condition.operationTemplate = element["operationTemplate"];
        condition.direction = element["direction"];
        if (condition.direction == undefined)
            condition.direction = true;
        return condition;
    }
}

var NestedDataStrategy;
(function (NestedDataStrategy) {
    NestedDataStrategy[NestedDataStrategy["None"] = 0] = "None";
    NestedDataStrategy[NestedDataStrategy["Concat"] = 1] = "Concat";
    NestedDataStrategy[NestedDataStrategy["Divide"] = 2] = "Divide";
})(NestedDataStrategy || (NestedDataStrategy = {}));
class Column {
    constructor() {
        this.index = 0;
        this.nestedDataStrategy = NestedDataStrategy.None;
        this.concatStrategy = ', ';
        this.displayText = "";
        this.supportsSorting = true;
        this.sortDirection = 2;
        this.filterEnabled = false;
        this.type = "data";
        this.format = "dd.MM.yyyy";
        this.isComputed = false;
        this.isActive = true;
        this.supportsGrouping = false;
        this.grouped = false;
        this.enableAddon = false;
        this.actionButtonDisplayText = "";
    }
    toggleSort() {
        if (this.sortDirection == 2)
            this.sortDirection = -1;
        this.sortDirection += 1;
    }
    getTemplate(item) {
        for (var index in this.conditions) {
            var condition = this.conditions[index];
            var data = this.extractValue(item, condition.field);
            if (condition.condition == data) {
                if (condition.operationCondition == undefined) {
                    return condition.template;
                }
                else if (item["__operationState"] == condition.operationCondition) {
                    return condition.operationTemplate;
                }
                if (condition.shouldOperationFallbackToDefaultTemplate) {
                    return condition.template;
                }
            }
        }
        return item[this.field];
    }
    extractValue(item, field, returnLength = true) {
        try {
            if (field == undefined)
                return "";
            if (field.indexOf('.') < 0) {
                if (item !== undefined && item.hasOwnProperty(field)) {
                    var data = item[field];
                    if (data instanceof Array) {
                        return returnLength ? data.length : data;
                    }
                    else {
                        return data;
                    }
                }
                else
                    return '';
            }
            var keyPath = field.split('.');
            var currentItem = item;
            for (var index in keyPath) {
                if (keyPath.hasOwnProperty(index)) {
                    var key = keyPath[index];
                    if (currentItem === undefined)
                        return "";
                    if (currentItem.hasOwnProperty(key)) {
                        currentItem = currentItem[key];
                    }
                    else
                        return "";
                }
            }
            if (currentItem instanceof Date) {
                return currentItem.toDateString();
            }
            if (currentItem instanceof Array) {
                return returnLength ? currentItem.length : currentItem;
            }
            return currentItem;
        }
        catch (error) {
            console.log(error);
            return "";
        }
    }
    static parseList(objects) {
        if (objects == undefined)
            return undefined;
        let items = new Array();
        let index = 0;
        objects.forEach(element => {
            var column = Column.parse(element);
            column.index = index++;
            items.push(column);
        });
        return items;
    }
    static parse(element) {
        if (element == undefined)
            return undefined;
        var column = new Column();
        if (element["isActive"] !== undefined)
            column.isActive = element["isActive"];
        column.field = element["field"];
        column.moneyInputValueField = element["moneyInputValueField"];
        column.moneyInputCurrencyIDField = element["moneyInputCurrencyIDField"];
        column.displayText = element["displayText"];
        column.actionButtonDisplayText = element["actionButtonDisplayText"];
        column.conditions = Condition.parseList(element["conditions"]);
        if (element["supportsSorting"] !== undefined)
            column.supportsSorting = element["supportsSorting"];
        if (element["sortDirection"] !== undefined)
            column.sortDirection = element["sortDirection"];
        column.sortKey = element["sortKey"];
        if (column.sortKey === undefined || column.sortKey.length === 0)
            column.sortKey = column.filterKey;
        column.join = element["join"];
        column.concatStrategy = element["concatStrategy"];
        column.isComputed = element["isComputed"];
        column.minWidth = element["min-width"];
        column.format = element["format"];
        column.nestedDataStrategy = element["nestedDataStrategy"];
        column.divisionField = element["divisionField"];
        column.typeConditions = element["typeConditions"];
        column.enableAddon = element["enableAddon"];
        column.addonSymbol = element["addonSymbol"];
        column.dateMaxValueField = element["dateMaxValueField"];
        column.dateMinValueField = element["dateMinValueField"];
        column.filterKey = element["filterKey"];
        column.filterEnabled = element["filterEnabled"];
        column.filter = Filter.parse(element["filter"]);
        if (column.filter != undefined && column.filter.filterKey == undefined) {
            column.filter.filterKey = column.filterKey;
            if ((column.filter.sourceTable == undefined || column.filter.sourceTable.length == 0) &&
                (column.join != undefined && column.join.length > 0)) {
                column.filter.sourceTable = column.join;
                var dotIndex = column.filter.sourceTable.lastIndexOf('.');
                if (dotIndex > -1) {
                    column.filter.sourceTable = column.filter.sourceTable.substring(dotIndex + 1, column.filter.sourceTable.length);
                }
            }
        }
        var type = element["type"];
        if (type != undefined) {
            column.type = type;
        }
        var selections = element["selections"];
        if (selections != undefined) {
            column.selections = selections;
        }
        return column;
    }
    static getIncludeStrings(columns) {
        var includeStrings = new Array();
        columns.forEach(column => {
            if (column.join != undefined) {
                includeStrings.push(column.join);
            }
        });
        return includeStrings;
    }
}

//import { BaseRepository } from 'app/repositories/base-repository';
//import { CommentComponent } from '../../components/comment/comment.component';
//import { DocumentListByownerComponent } from 'app/components/shared/document/document-list-byowner/document-list-byowner.component';
let DataTableComponent = class DataTableComponent /*extends BaseComponent*/ {
    constructor(dialog, router, route /*, private userDefinedFilterRepo: UserDefinedFilterRepository*/) {
        this.dialog = dialog;
        this.router = router;
        this.route = route;
        this.postInProgress = false;
        this.identifier = UUID.UUID();
        this.commentsEnabled = false;
        this.documentsEnabled = false;
        this.uiSelectionEnabled = false;
        this.baseURL = "";
        this.emptyResultTemplateShownInPager = true;
        this.columnFilterEnabled = false;
        this.hasFilters = false;
        this.items = new Array();
        this.pageCount = 50;
        this.pageIndex = 0;
        this.itemCount = 0;
        this.configPath = "";
        this.configurationLoaded = false;
        this.onConfigurationLoaded = new EventEmitter();
        this.onFilterUpdated = new EventEmitter();
        this.onSortPreferenceChanged = new EventEmitter();
        this.onGroupPreferenceChanged = new EventEmitter();
        this.onPageChanged = new EventEmitter();
        this.onPageCountChanged = new EventEmitter();
        this.onUserDefinedFilterSelected = new EventEmitter();
        this.onInsertFormRequested = new EventEmitter();
        this.onEntitySelected = new EventEmitter();
        this.onRowSelected = new EventEmitter();
        this.onColumnSelected = new EventEmitter();
        this.onSelectionChanged = new EventEmitter();
        this.selectedAction = undefined;
        this.isFullScreen = false;
        this.isScrollable = true;
        this.isCompressed = true;
        this.containsNestedData = false;
        this.dataTableStorage = new DataTableStorageModel();
        this.userDefinedFilter = new UserDefinedFilterModel();
        this.comboboxSettings = new ComboboxSettings();
        this.comboboxStyles = new ComboboxStyles();
        this.filterManagementPageMode = FilterManagementPageMode.List;
        this.rowSelectionEnabled = false;
        this.itemSelectionChanged = false;
        this.allSelected = false;
        this.optionsMenuVisible = false;
        // Keyboard actions
        this.activeRow = undefined;
        this.activeColumn = undefined;
        this.columnCount = undefined;
        this.rowCount = undefined;
        this.rowToBeSelected = undefined;
        this.columnToBeSelected = undefined;
        this.tableSettingMode = TableSettingMode.Default;
        //super();
        //this.baseURL = new BaseRepository(undefined).apiRoot;
        //let userType = LocalStorageHandler.getUserType();
        //this.isAdmin = userType == 5;
    }
    get dataTableID() {
        return this._dataTableID;
    }
    set dataTableID(v) {
        this.dataTableStorage.dataTableID = v;
        this._dataTableID = v;
    }
    ngOnInit() {
        this.dataTableStorage = LocalStorageHandler.getDataTableSettings(this.dataTableID);
        this.setDataTableSettings(this.dataTableStorage, false);
        this.setComboboxSettings();
        this.loading = true;
        this.getUserDefinedFilters();
    }
    setComboboxSettings() {
        this.comboboxSettings.enableSearch = true;
        this.comboboxSettings.lazyLoadEnabled = true;
        this.comboboxSettings.selectionLimit = 1;
        this.comboboxSettings.closeOnSelect = true;
    }
    setDataTableSettings(dataTableSetting, triggerOnFilterUpdated = true) {
        this.dataTableStorage = dataTableSetting;
        if (dataTableSetting.pageCount == undefined)
            dataTableSetting.pageCount = this.pageCount;
        if (dataTableSetting.pageIndex == undefined)
            dataTableSetting.pageIndex = this.pageIndex;
        this.filters = dataTableSetting.filters;
        if (triggerOnFilterUpdated)
            this.onFilterUpdated.emit({ filter: this.filters, pageIndex: dataTableSetting.pageIndex, pageCount: dataTableSetting.pageCount });
        this.isScrollable = dataTableSetting.isScrollable;
        this.isFullScreen = dataTableSetting.isFullScreen;
        this.isCompressed = dataTableSetting.isCompressed;
        this.pageIndex = dataTableSetting.pageIndex;
        this.pageCount = dataTableSetting.pageCount;
        this.activateColumns();
        this.setActiveColumns();
    }
    clearFilters() {
        if (this.columns) {
            this.columns.forEach(column => {
                if (column) {
                    if (column.filterEnabled && column.filter) {
                        column.filter.value = undefined;
                        if (column.filter.type == 4 && column.filter.items) {
                            column.filter.items.forEach(item => {
                                item.setSelected(false);
                            });
                            column.filter.combobox.clearSelection();
                        }
                    }
                }
            });
        }
        this.hasFilters = false;
        this.filters = undefined;
        this.onFilterUpdated.emit({ filter: undefined, pageIndex: this.dataTableStorage.pageIndex, pageCount: this.dataTableStorage.pageCount });
    }
    pageChanged($event) {
        this.dataTableStorage.pageIndex = +$event ? +$event : 0;
        LocalStorageHandler.setDataTableSettings(this.dataTableStorage);
        if (this.onPageChanged !== undefined && this.onPageChanged.observers.length > 0) {
            this.loading = true;
            this.onPageChanged.emit($event);
        }
    }
    pageCountChanged($event) {
        //this.loading = true;
        this.pageCount = +$event;
        this.dataTableStorage.pageCount = this.pageCount;
        LocalStorageHandler.setDataTableSettings(this.dataTableStorage);
        if (+$event == 0) {
            Utils.showConfirmDialog(this.dialog, { panelClass: 'modal-dialog', data: { contentText: 'Veri sayısı çok fazla olduğundan excel halinde indirilecektir.Onaylıyor musunuz?' } }, this.confirm, this.decline, this);
            return;
        }
        this.onPageCountChanged.emit($event);
    }
    actionTriggered(action, item) {
        this.closeContextMenu();
        item["__operationState"] = "activating";
        action.action.call(this.context, item);
    }
    loadConfig(loader) {
        loader.then(tableConfig => {
            this.tableConfig = tableConfig;
            this.applyConfig(tableConfig);
        });
    }
    applyConfig(tableConfig) {
        this.tableConfig = tableConfig;
        if (tableConfig === undefined) {
            return;
        }
        this.enabled = tableConfig["enabled"];
        this.commentsEnabled = tableConfig["commentsEnabled"];
        this.documentsEnabled = tableConfig["documentsEnabled"];
        this.columnFilterEnabled = tableConfig["columnFilterEnabled"];
        this.columns = Column.parseList(tableConfig["columns"]);
        this.activateColumns();
        this.changeFilterState();
        this.actionColumn = Column.parse(tableConfig["actionColumn"]);
        this.configurationLoaded = true;
        this.onConfigurationLoaded.emit({ includeStrings: Column.getIncludeStrings(this.columns) });
        this.setActiveColumns();
        var division = this.getDivisionColumns();
        this.containsNestedData = division !== undefined && division.length > 0;
    }
    activateColumns() {
        if (this.dataTableStorage.columns != undefined && this.columns != undefined) {
            this.columns.forEach(column => {
                this.dataTableStorage.columns.forEach((dtColumn, i) => {
                    if (column.field == dtColumn.field) {
                        column.index = i;
                        column.isActive = dtColumn.isActive;
                    }
                });
            });
            this.columns = this.columns.sort((a, b) => a.index - b.index);
        }
        else {
            this.saveDataTableColumns();
        }
    }
    getAllItems() {
        return this.items;
    }
    confirm() {
        this.onPageCountChanged.emit(0);
    }
    decline() {
        return;
    }
    setItems(items, itemCount, pageNumber, userDefinedFilters = undefined, predefinedFilter = undefined) {
        // if (this.pageCount === 0) {
        //   Utils.showConfirmDialog(this.dialog, { panelClass: 'modal-dialog', data: { contentText: 'Veri sayısı çok fazla olduğundan excel halinde indirilecektir.Onaylıyor musunuz?' } }, this.confirm, this.decline, this);
        //   return;
        // }
        if (this.pageCount == 0) {
            this.exportItemsToExcel(items);
            return;
        }
        this.setPredefinedFilter(predefinedFilter);
        this.setUserDefinedFilters(userDefinedFilters);
        this.items = items;
        var division = this.getDivisionColumns();
        this.items.forEach(item => {
            var rowCounts = new Array();
            division.forEach(column => {
                if (column.divisionField !== undefined) {
                    var value = this.extractRawValue(item, column.divisionField, ',', false);
                    if (value instanceof Array) {
                        var length = value.length > 0 ? value.length : 1;
                        item["_" + column.divisionField + "Count"] = length;
                        rowCounts.push(length);
                    }
                }
            });
            var lcm = this.range_lcm(rowCounts);
            item["_rowCount"] = lcm;
            item["_rowIterator"] = Array(lcm).fill(1);
        });
        this.placeboRowHeight = 380;
        if (this.items) {
            this.placeboRowHeight = 380 - this.items.length * 50;
            this.rowCount = this.items.length;
        }
        this.itemCount = itemCount;
        this.pageIndex = pageNumber;
        this.renderItems();
        setTimeout(() => {
            if (itemCount > 0 && this.pager !== undefined) {
                this.pager.flushPager(itemCount);
            }
            this.loading = false;
        }, 10);
    }
    renderItems(items = undefined) {
        if (items == undefined)
            items = this.items;
        // console.log('Render Started: ' + new Date())
        var datePipe = new DatePipe('tr');
        for (let index = 0; index < items.length; index++) {
            const item = items[index];
            item["_render"] = new Object();
            var renderObject = item["_render"];
            renderObject["_deletedCondition"] = this.getDeletedCondition(item, index);
            renderObject["_uiSelectionEnabled"] = new Array();
            renderObject["_tdVisible"] = new Array();
            var rowIteratorx = item["_rowIterator"];
            rowIteratorx.forEach((x, rowIndex) => {
                renderObject["_uiSelectionEnabled"].push(this.uiSelectionEnabled && (rowIndex % item._rowCount == 0));
                renderObject["_tdVisible"].push(rowIndex % item._rowCount == 0);
            });
            for (const column of this.columns) {
                if (column.type == 'condition') {
                    renderObject["_template_" + column.index] = column.getTemplate(item);
                }
                else if (column.nestedDataStrategy == 2) {
                    var dataList = new Array();
                    var rowIterator = item["_rowIterator"];
                    rowIterator.forEach((x, rowIndex) => {
                        var data = new Object();
                        var displayText = this.extractDivisionValue(item, column, rowIndex);
                        if (column.type == 'date')
                            displayText = datePipe.transform(data, column.format);
                        data["displayText"] = displayText;
                        data["rowSpan"] = item._rowCount / item['_' + column.divisionField + 'Count'];
                        data["rowSpanEnabled"] = rowIndex % (item._rowCount / item['_' + column.divisionField + 'Count']) == 0;
                        renderObject[column.field] = data;
                        dataList.push(data);
                    });
                    renderObject[column.field] = dataList;
                }
                else {
                    var data = this.extractValue(item, column);
                    if (column.type == 'date')
                        data = datePipe.transform(data, column.format);
                    renderObject[column.field] = data;
                }
            }
        }
        // console.log('Render Ended: ' + new Date())
    }
    createContextMenu() {
    }
    setActiveColumns() {
        if (this.columns == undefined || this.columns.length == 0)
            this.activeColumns = new Array();
        else
            this.activeColumns = this.columns.filter(row => row.isActive == true);
        this.saveDataTableColumns();
        this.columnCount = this.activeColumns.length;
    }
    saveDataTableColumns() {
        if (this.dataTableID == undefined)
            return;
        if (this.dataTableStorage.dataTableID == undefined)
            this.dataTableStorage.dataTableID = this.dataTableID;
        if (this.columns == undefined)
            return;
        this.dataTableStorage.columns = new Array();
        this.columns.forEach((column, i) => {
            let col = new Column();
            col.index = i;
            col.isActive = column.isActive;
            col.filterKey = column.filterKey;
            col.displayText = column.displayText;
            col.field = column.field;
            this.dataTableStorage.columns.push(col);
        });
        LocalStorageHandler.setDataTableSettings(this.dataTableStorage);
    }
    getDivisionColumns() {
        if (this.columns == undefined || this.columns.length == 0)
            return new Array();
        return this.columns.filter(row => row.nestedDataStrategy == NestedDataStrategy.Divide);
    }
    setFilter(filter, items, operator = undefined) {
        this.columns.forEach(element => {
            if (element.filterEnabled == true && element.filter.type == 4) {
                if (element.filterKey == filter) {
                    element.filter.items = items;
                    element.filter.loading = false;
                    if (operator) {
                        element.filter.operation = operator;
                    }
                }
            }
        });
    }
    searchInputChanged() {
        this.onFilterChanged();
    }
    generalSearchInputChanged($event) {
    }
    // this.tableStorageManager.saveFilters(this.table, this.table.name);
    onFilterChanged() {
        this.pageIndex = 0;
        this.dataTableStorage.pageIndex = 0;
        let filters = new Array();
        this.columns.forEach(element => {
            if (element.filterEnabled == true && element.filter.value != undefined) {
                if (element.filter.type == 2 && element.filter.operation == undefined) {
                    var criteria = new SearchCriteria();
                    criteria.operator = '>=';
                    criteria.key = element.filterKey;
                    criteria.value = element.filter.value;
                    filters.push(criteria);
                    criteria = new SearchCriteria();
                    criteria.operator = '<';
                    criteria.key = element.filterKey;
                    var selectedDate = new Date(element.filter.value);
                    selectedDate.setDate(selectedDate.getDate() + 1);
                    criteria.value = selectedDate;
                    filters.push(criteria);
                }
                else {
                    var criteria = new SearchCriteria();
                    criteria.operator = element.filter.operation;
                    criteria.key = element.filterKey;
                    if (element.filter.valueMap !== undefined) {
                        if (element.filter.valueMap.containsKey(element.filter.value)) {
                            criteria.value = element.filter.valueMap[element.filter.value];
                            filters.push(criteria);
                        }
                    }
                    else {
                        criteria.value = element.filter.value;
                        filters.push(criteria);
                    }
                }
                this.hasFilters = true;
            }
        });
        // this.tableStorageManager.saveFilters(this.table, this.table.name);
        this.filters = filters;
        this.dataTableStorage.filters = filters;
        LocalStorageHandler.setDataTableSettings(this.dataTableStorage);
        this.onFilterUpdated.emit({ filter: filters, pageIndex: this.dataTableStorage.pageIndex, pageCount: this.dataTableStorage.pageCount });
    }
    changeFilterState() {
        if (this.dataTableStorage.filters && this.activeColumns) {
            this.dataTableStorage.filters.forEach(filter => {
                this.activeColumns.forEach(column => {
                    if (column.filterEnabled == true && column.filterKey == filter.key) {
                        if (column.filter.type == 4) {
                            if (column.filter.items && column.filter.items.length > 0 && filter.value && filter.value.length > 0) {
                                filter.value.forEach(val => {
                                    let found = column.filter.items.find(item => item.value == val);
                                    found.setSelected(true);
                                });
                            }
                        }
                        else {
                            column.filter.value = filter.value;
                        }
                        this.hasFilters = true;
                    }
                });
            });
            this.dataTableFilters.forEach(x => x.updateFilters());
        }
    }
    sort(column) {
        if (column.supportsSorting != true)
            return;
        column.toggleSort();
        let sortPreferences = new Dictionary(null);
        this.columns.forEach(element => {
            if (element.supportsSorting == true && !(element.sortDirection == undefined || element.sortDirection == 2)) {
                let sortKey = element.sortKey;
                if (sortKey == undefined || sortKey.length == 1)
                    sortKey = element.filterKey;
                sortPreferences.add(sortKey, element.sortDirection);
            }
        });
        this.onSortPreferenceChanged.emit({ sortPreferences: sortPreferences });
    }
    group(column) {
        if (column.supportsGrouping != true)
            return;
        column.grouped = !column.grouped;
        let groupPreferences = new Dictionary(null);
        this.columns.forEach(element => {
            if (element.supportsSorting == true && !(element.sortDirection == undefined || element.sortDirection == 2)) {
                groupPreferences.add(element.sortKey, element.sortDirection);
            }
        });
        this.onGroupPreferenceChanged.emit({ groupPreferences: groupPreferences });
    }
    extractDivisionValue(item, column, index) {
        var value = this.extractRawValue(item, column.divisionField, ',', false);
        if (value == undefined)
            return "";
        var count = item['_' + column.divisionField + 'Count'];
        if (count == undefined)
            count = 1;
        if (item._rowCount == undefined)
            item._rowCount = 1;
        var nestedItem = value[index / (item._rowCount / (count))];
        return this.extractValue(nestedItem, column);
    }
    extractValue(item, column) {
        if (column.isComputed) {
            if (this.computeProperyForItem !== undefined) {
                return this.computeProperyForItem.call(this.context, item, column.field);
            }
            else {
                return "";
            }
        }
        var extracted = this.extractRawValue(item, column.field, column.concatStrategy);
        if (extracted !== undefined && extracted instanceof String && extracted.length > 100) {
            extracted = extracted.slice(0, 97) + "...";
        }
        return extracted;
    }
    extractRawValue(item, field, separator = ', ', returnLength = true) {
        if (field.indexOf('.') < 0) {
            if (item !== undefined && item.hasOwnProperty(field)) {
                var data = item[field];
                if (data instanceof Array) {
                    return returnLength ? data.length : data;
                }
                else {
                    return data;
                }
            }
            else
                return '';
        }
        var keyPath = field.split('.');
        var currentItem = item;
        for (var index in keyPath) {
            if (keyPath.hasOwnProperty(index)) {
                var key = keyPath[index];
                if (currentItem === undefined)
                    return "";
                if (currentItem.hasOwnProperty(key)) {
                    currentItem = currentItem[key];
                    if (currentItem instanceof Array) {
                        var remainingKeyPath = keyPath.slice(+index + 1, keyPath.length).join(".");
                        var nested = new Array();
                        currentItem.forEach(element => {
                            nested.push(this.extractRawValue(element, remainingKeyPath));
                        });
                        return nested.join(separator);
                    }
                }
                else
                    return "";
            }
        }
        if (currentItem instanceof Date) {
            return currentItem.toDateString();
        }
        if (currentItem instanceof Array) {
            return returnLength ? currentItem.length : currentItem;
        }
        return currentItem;
    }
    presentInsertForm() {
        this.onInsertFormRequested.emit();
    }
    getDeletedCondition(item, rowIndex) {
        var backgroundColor = (rowIndex % 2 == 0) ? "#f3f6f8" : 'transparent';
        if (this.containsNestedData == true) {
            backgroundColor = item === this.activeItem ? "#f3f6f8" : backgroundColor;
        }
        else {
            backgroundColor = (rowIndex == this.activeRow) ? "#f3f6f8" : backgroundColor;
        }
        if (item.hasOwnProperty("isDeleted") && item["isDeleted"] == true) {
            return { 'text-decoration': 'line-through', 'background-color': backgroundColor };
        }
        else if (item.hasOwnProperty("isActive") && item["isActive"] == false) {
            return { 'text-decoration': 'line-through', 'background-color': backgroundColor };
        }
        return { 'background-color': backgroundColor };
    }
    clickedOutside($event) {
        this.closeContextMenu();
    }
    onWindowScroll(event) {
        this.closeContextMenu();
    }
    onScroll(event) {
        this.closeContextMenu();
    }
    onMouseWheel(event) {
        this.closeContextMenu();
    }
    toggleActionsDropdown(event, item) {
        event.preventDefault();
        event.stopPropagation();
        this.activeContextItem = item;
        if (event instanceof MouseEvent) {
            var relativeContainer = document.querySelector(".data-table-container[identifier='" + this.identifier + "']");
            var content = document.getElementById('action_menu_content_' + this.identifier);
            content.style.right = 'auto';
            content.style.bottom = 'auto';
            content.style.top = event.clientY - relativeContainer.getBoundingClientRect().top + "px";
            content.style.left = event.clientX - relativeContainer.getBoundingClientRect().left + 'px';
            var actionMenuContainer = document.getElementById('action_menu_container_' + this.identifier);
            if (actionMenuContainer) {
                actionMenuContainer.classList.add('open');
            }
        }
    }
    closeContextMenu() {
        var actionMenuContainer = document.getElementById('action_menu_container_' + this.identifier);
        if (actionMenuContainer) {
            actionMenuContainer.classList.remove('open');
        }
    }
    exportPDF() {
        alert("PDF Export is not implemented yet.");
    }
    exportExcel() {
        this.exportItemsToExcel(this.items);
    }
    prepareItems(items) {
        var division = this.getDivisionColumns();
        items.forEach(item => {
            var rowCounts = new Array();
            division.forEach(column => {
                if (column.divisionField !== undefined) {
                    var value = this.extractRawValue(item, column.divisionField, ',', false);
                    if (value instanceof Array) {
                        var length = value.length > 0 ? value.length : 1;
                        item["_" + column.divisionField + "Count"] = length;
                        rowCounts.push(length);
                    }
                }
            });
            var lcm = this.range_lcm(rowCounts);
            item["_rowCount"] = lcm;
            item["_rowIterator"] = Array(lcm).fill(1);
        });
        this.renderItems(items);
    }
    // NestedDataStrategy
    // None = 0,
    // Concat = 1,
    // Divide = 2
    exportItemsToExcel(items) {
        if (items != undefined && items.length > 0) {
            this.prepareItems(items);
            var data = [];
            items.forEach(item => {
                var row = new Object();
                this.activeColumns.forEach(column => {
                    var value = undefined;
                    if (column.type == 'condition') {
                        // Need excel template as well
                        value = column.getTemplate(item);
                        if (value != undefined) {
                            var regex = /(<([^>]+)>)/ig;
                            value = value.replace(regex, "");
                        }
                    }
                    else if (column.type == 'data') {
                        value = this.extractValue(item, column);
                    }
                    else if (column.type === 'date') {
                        // Need pipe here, use the one below.
                        value = this.extractValue(item, column);
                        //extractValue(item, column) | date:column.format
                    }
                    else if (column.isComputed) {
                        if (this.computeProperyForItem !== undefined) {
                            value = this.computeProperyForItem.call(this.context, item, column.field);
                            var regex = /(<([^>]+)>)/ig;
                            value = value.replace(regex, "");
                        }
                    }
                    row[column.displayText] = value;
                });
                if (item._rowCount > 0) {
                    item._rowIterator.forEach((element, index) => {
                        var innerRow = new Object();
                        Object.assign(innerRow, row);
                        this.activeColumns.forEach(column => {
                            if (column.nestedDataStrategy > 0)
                                innerRow[column.displayText] = item._render[column.field][index]['displayText'];
                        });
                        data.push(innerRow);
                    });
                }
                else {
                    data.push(row);
                }
            });
            var opts = [{ sheeitd: "Sheet 1", header: true }];
            utils.isBrowserify = false;
            utils.global.XLSX = XLSX;
            alasql("SELECT INTO XLSX ('Export.xlsx',?) FROM ?", [opts, [data]]);
        }
    }
    /*
     
        let DataForSheet1: any[] = [];
        DataForSheet1.push({ FName: "John", LName: "Kennedy" });
        DataForSheet1.push({ FName: "Ajay", LName: "Kumar" });
        DataForSheet1.push({ FName: "Deepak", LName: "Reddy" });
        DataForSheet1.push({ FName: "Kiran", LName: "Kumar" });
     
        let DataForSheet2: any[] = [];
        DataForSheet2.push({ Name: "John", Mark: 100 });
        DataForSheet2.push({ Name: "Ajay", LName: 85 });
        DataForSheet2.push({ Name: "Deepak", LName: 70 });
        DataForSheet2.push({ Name: "Kiran", LName: 90 });
     
        var opts = [{ sheeitd: "Sheet 1", header: true }, { sheeitd: "Sheet 2", header: true }]
     
    alasql['utils'].isBrowserify = false;
    alasql['utils'].global.XLSX = XLSX;
     
        alasql("SELECT INTO XLSX ('Students.xlsx',?) FROM ?", [opts, [DataForSheet1, DataForSheet2]]);
    //This piece of code will download excel sheet with two sheets
     
        alasql("SELECT TOP 2 Name as [Student Name], Mark as [Subject Marks] INTO XLSX ('Marks.xlsx',{headers:true}) FROM ?", [DataForSheet2]);
    //This piece of code will download excel sheet with one sheet, and if you notice here we could change the column name also.
     
    //
     
     
    */
    toggleScroll() {
        this.isScrollable = !this.isScrollable;
        this.dataTableStorage.isScrollable = this.isScrollable;
        LocalStorageHandler.setDataTableSettings(this.dataTableStorage);
    }
    toggleCompactMode() {
        this.isCompressed = !this.isCompressed;
        this.dataTableStorage.isCompressed = this.isCompressed;
        LocalStorageHandler.setDataTableSettings(this.dataTableStorage);
    }
    toggleSize(event) {
        this.isFullScreen = !this.isFullScreen;
        this.dataTableStorage.isFullScreen = this.isFullScreen;
        LocalStorageHandler.setDataTableSettings(this.dataTableStorage);
        const eventTarget = event.target;
        if (eventTarget !== undefined && eventTarget !== null) {
            const container = eventTarget.closest(this.isFullScreen ? '.container' : '.container-fluid');
            if (container !== undefined && container !== null) {
                if (!container.classList.contains('margin-t-10')) {
                    container.classList.add('margin-t-10');
                }
                container.classList.remove(this.isFullScreen ? 'container' : 'container-fluid');
                container.classList.add(this.isFullScreen ? 'container-fluid' : 'container');
            }
        }
    }
    itemClicked(item) {
        if (this.rowSelectionEnabled == true) {
            item._selected = !item._selected;
            // TODO : Check for multiple event triggers.
            this.onSelectionChanged.emit(this.getSelectedItems());
        }
    }
    getSelectedItems() {
        return this.items.filter(x => x._selected == true);
    }
    checkedChanged(item) {
        var selectedItems = this.getSelectedItems();
        this.onSelectionChanged.emit(selectedItems);
        this.itemSelectionChanged = true;
        this.allSelected = selectedItems.length == this.items.length;
    }
    toggleAllSelection() {
        if (this.itemSelectionChanged) {
            this.itemSelectionChanged = false;
            return;
        }
        this.items.forEach(item => { item._selected = this.allSelected; });
    }
    onRightClick($event, item) {
        this.toggleActionsDropdown($event, item);
        return false;
    }
    toggleOptionsMenu() {
        this.optionsMenuVisible = !this.optionsMenuVisible;
    }
    closeOptionsMenu() {
        this.optionsMenuVisible = false;
    }
    shiftColumn(column, shift) {
        var index = this.columns.indexOf(column);
        var targetIndex = index + shift;
        this.columns[index] = this.columns[targetIndex];
        this.columns[targetIndex] = column;
        this.setActiveColumns();
    }
    activeColumnConfigChanged() {
        this.setActiveColumns();
    }
    // Returns true if selection changed, false if not. Undefined if configuration is not set.
    columnSelected(row, column, item) {
        if (this.rowCount === undefined || this.columnCount === undefined) {
            return undefined;
        }
        this.activeItem = item;
        if (!this.rowSelected(row) && this.activeColumn == column)
            return false;
        this.activeColumn = column;
        this.onColumnSelected.emit(this.activeColumn);
    }
    // Returns true if selection changed, false if not. Undefined if configuration is not set
    rowSelected(row) {
        if (this.rowCount === undefined || this.columnCount === undefined) {
            return undefined;
        }
        if (this.activeRow == row)
            return false;
        this.activeRow = row;
        this.onRowSelected.emit(this.activeRow);
        this.onEntitySelected.emit(this.items[this.activeRow]);
        return true;
    }
    keyboardInput($event) {
        var code = event["code"];
        if (code == "ArrowRight" || code == "ArrowLeft" || code == "ArrowUp" || code == "ArrowDown" || code == "Space") {
            if (code == "Space") {
                this.activeColumn = this.columnToBeSelected;
                this.activeRow = this.rowToBeSelected;
                this.rowToBeSelected = undefined;
                this.columnToBeSelected = undefined;
                return;
            }
            var currentRow = (this.rowToBeSelected != undefined) ? this.rowToBeSelected : ((this.activeRow != undefined) ? this.activeRow : 0);
            var currentColumn = (this.columnToBeSelected != undefined) ? this.columnToBeSelected : ((this.activeColumn != undefined) ? this.activeColumn : 0);
            if (code == "ArrowRight") {
                currentColumn = currentColumn + 1;
            }
            else if (code == "ArrowLeft") {
                currentColumn = currentColumn - 1;
            }
            else if (code == "ArrowUp") {
                currentRow = currentRow - 1;
            }
            else if (code == "ArrowDown") {
                currentRow = currentRow + 1;
            }
            currentRow = Math.max(0, currentRow);
            currentColumn = Math.max(0, currentColumn);
            currentRow = Math.min(currentRow, this.rowCount - 1);
            currentColumn = Math.min(currentColumn, this.columnCount - 1);
            this.rowToBeSelected = currentRow;
            this.columnToBeSelected = currentColumn;
            event.preventDefault();
            event.stopPropagation();
        }
        // ArrowRight
        // ArrowLeft
        // ArrowUp
        // ArrowDown
        // this.keyboardEvent = event;
        // this.altKey = event.altKey;
        // this.charCode = event.charCode;
        // this.code = event.code;
        // this.ctrlKey = event.ctrlKey;
        // this.keyCode = event.keyCode;
        // this.keyIdentifier = event.keyIdentifier;
        // this.metaKey = event.metaKey;
        // this.shiftKey = event.shiftKey;
        // this.timeStamp = event.timeStamp;
        // this.type = event.type;
        // this.which = event.which;
    }
    // End of keyboard actions.
    // Euclid algorithm for Greates Common Divisor
    gcd(a, b) {
        return !b ? a : this.gcd(b, a % b);
    }
    // Least Common Multiple function
    lcm(a, b) {
        return a * (b / this.gcd(a, b));
    }
    // LCM of all numbers in the range of arr=[a, b]
    range_lcm(arr) {
        var result = arr[0];
        arr.forEach(element => {
            result = this.lcm(result, element);
        });
        return result;
    }
    setPredefinedFilter(filter) {
        if (filter != undefined && filter.length > 0) {
            this.dataTableStorage = DataTableStorageModel.parse(JSON.parse(filter));
            this.changeFilterState();
            this.setDataTableSettings(this.dataTableStorage, false);
        }
    }
    turnDefaultFilterActionClicked() {
        let dtSetting = LocalStorageHandler.removeDataTableFilter(this.dataTableID);
        this.setDataTableSettings(dtSetting);
    }
    editUserDefinedFilterActionClicked(userDefinedFilter) {
        userDefinedFilter.selected = true;
        this.activeUserDefinedFilter = userDefinedFilter.owner;
        this.filterManagementPageMode = FilterManagementPageMode.Update;
    }
    addNewFilterButtonClicked() {
    }
    cancelUserDefinedFilterActionClicked(userDefinedFilter) {
        if (this.filterManagementPageMode == FilterManagementPageMode.Insert) {
            let index = this.userDefinedFilters.indexOf(userDefinedFilter);
            if (index > -1)
                this.userDefinedFilters.splice(index, 1);
        }
        this.userDefinedFilters.map(x => x.selected = false);
        this.activeUserDefinedFilter = undefined;
        this.filterManagementPageMode = FilterManagementPageMode.List;
    }
    activateActionClicked(userDefinedFilter) {
    }
    setUserDefinedFilters(filters) {
    }
    selectedUserDefinedFilterChange(event) {
        if (event != undefined && event.length > 0 && event[0] != undefined) {
            this.onUserDefinedFilterSelected.emit(event[0].value);
            // let userDefinedFilter = event[0].owner;
            // this.dataTableStorage = DataTableStorageModel.parse(JSON.parse(userDefinedFilter.filterJson));
            // this.changeFilterState();
            // this.setDataTableSettings(this.dataTableStorage);
        }
    }
    editTable() {
        const navigationExtras = { relativeTo: this.route, state: { 'tableConfig': this.tableConfig, 'configPath': this.configPath } };
        this.router.navigate(['/administration/data-table-editor'], navigationExtras);
    }
    openFilterPopup(userDefinedFilterPopupTemplate) {
        this.tableSettingMode = TableSettingMode.FilterManagement;
        // var dialogRef = this.dialog.open(userDefinedFilterPopupTemplate, { panelClass: 'modal-dialog' });
    }
    switchTableSettingMode() {
        this.tableSettingMode = this.tableSettingMode == TableSettingMode.FilterManagement ? TableSettingMode.Default : TableSettingMode.FilterManagement;
    }
    deleteUserDefinedFilterActionClicked(userDefinedFilter) {
        // this.startProgress("");
        // this.userDefinedFilterRepo.delete(userDefinedFilter.value)
        //   .then(response => {
        //     if (!response.hasError()) {
        //       let index = this.userDefinedFilters.indexOf(userDefinedFilter);
        //       if (index > -1)
        //         this.userDefinedFilters.splice(index, 1);
        //       this.successMessage = "İşlem başarılı bir şekilde tamamlandı.";
        //       this.filterManagementPageMode = FilterManagementPageMode.List;
        //       this.stopProgress();
        //     } else {
        //       this.hasError = true;
        //       this.errorMessage = response.getFirstErrorMessage();
        //       this.stopProgress();
        //     }
        //   })
        //   .catch(error => {
        //     this.hasError = true;
        //     this.errorMessage = `Sunucuyla bağlantı kurulurken hata oluştu. Lütfen tekrar deneyiniz. Hata detayı: ${error}`;
        //     this.stopProgress();
        //   });
    }
    saveUserDefinedFilter() {
        // if (!this.activeUserDefinedFilter.name) {
        //   this.errorMessage = "Filtre adı boş bırakılamaz";
        //   return;
        // }
        // let request: Promise<ResponseModel<UserDefinedFilterModel>>;
        // if (!this.activeUserDefinedFilter.id) {
        //   this.activeUserDefinedFilter.filterJson = JSON.stringify(this.dataTableStorage);
        //   this.activeUserDefinedFilter.userID = +LocalStorageHandler.getUserId();
        //   this.activeUserDefinedFilter.dataTableID = this.dataTableID;
        //   request = this.userDefinedFilterRepo.post(
        //     this.activeUserDefinedFilter
        //   );
        // } else if (this.activeUserDefinedFilter.id) {
        //   request = this.userDefinedFilterRepo.put(
        //     this.activeUserDefinedFilter
        //   );
        // }
        // this.startProgress("");
        // request
        //   .then(response => {
        //     if (!response.hasError()) {
        //       let foundItem: FilterItem;
        //       if (!this.activeUserDefinedFilter.id)
        //         foundItem = this.userDefinedFilters.find(x => !x.value);
        //       else
        //         foundItem = this.userDefinedFilters.find(x => x.value == this.activeUserDefinedFilter.id);
        //       let index = this.userDefinedFilters.indexOf(foundItem);
        //       if (index > -1)
        //         this.userDefinedFilters.splice(index, 1);
        //       this.userDefinedFilters.push(response.entity.asFilterItem(response.entity));
        //       this.successMessage = "İşlem başarılı bir şekilde tamamlandı.";
        //       this.filterManagementPageMode = FilterManagementPageMode.List;
        //       this.stopProgress();
        //     } else {
        //       this.hasError = true;
        //       this.errorMessage = response.getFirstErrorMessage();
        //       this.stopProgress();
        //     }
        //   })
        //   .catch(error => {
        //     this.hasError = true;
        //     this.errorMessage = `Sunucuyla bağlantı kurulurken hata oluştu. Lütfen tekrar deneyiniz. Hata detayı: ${error}`;
        //     this.stopProgress();
        //   });
    }
    saveTableSettings() {
        // if (!this.userDefinedFilter.name) {
        //   this.errorMessage = "Ad boş bırakılamaz";
        //   return;
        // }
        // this.userDefinedFilter.filterJson = JSON.stringify(this.dataTableStorage);
        // this.userDefinedFilter.userID = +LocalStorageHandler.getUserId();
        // this.userDefinedFilter.dataTableID = this.dataTableID;
        // const request = this.userDefinedFilterRepo.post(
        //   this.userDefinedFilter
        // );
        // this.startProgress("");
        // request
        //   .then(response => {
        //     if (!response.hasError()) {
        //       this.userDefinedFilter = response.entity;
        //       this.successMessage = "İşlem başarılı bir şekilde tamamlandı.";
        //       this.dialog.closeAll();
        //       this.stopProgress();
        //     } else {
        //       this.hasError = true;
        //       this.errorMessage = response.getFirstErrorMessage();
        //       this.stopProgress();
        //     }
        //   })
        //   .catch(error => {
        //     this.hasError = true;
        //     this.errorMessage = `Sunucuyla bağlantı kurulurken hata oluştu. Lütfen tekrar deneyiniz. Hata detayı: ${error}`;
        //     this.stopProgress();
        //   });
    }
    getUserDefinedFilters() {
        // let userID = +LocalStorageHandler.getUserId();
        // if (!userID) {
        //   return;
        // }
        // let searchCriteria = new SearchCriteria();
        // searchCriteria.key = "DataTableID";
        // searchCriteria.value = [this.dataTableID];
        // this.startProgress("");
        // this.userDefinedFilterRepo.getByUser(userID, 0, 0, false, [searchCriteria])
        //   .then(response => {
        //     if (!response.hasError()) {
        //       this.userDefinedFilters = response.entities.asFilter();
        //       this.filterManagementPageMode = FilterManagementPageMode.List;
        //       this.stopProgress();
        //     } else {
        //       this.hasError = true;
        //       this.errorMessage = response.getFirstErrorMessage();
        //       this.stopProgress();
        //     }
        //   })
        //   .catch(error => {
        //     this.hasError = true;
        //     this.errorMessage = `Sunucuyla bağlantı kurulurken hata oluştu. Lütfen tekrar deneyiniz. Hata detayı: ${error}`;
        //     this.stopProgress();
        //   });
    }
    showCommentsButtonClicked(item) {
        // const dialogRef = this.dialog.open(CommentComponent, { panelClass: 'modal-dialog' });
        // dialogRef.componentInstance.sourceGuid = item.rowguid;
        // dialogRef.componentInstance.sourceId = 2;
        // dialogRef.componentInstance.showLikeButton = true;
        // dialogRef.afterClosed().subscribe(result => {
        // });
    }
    showDocumentsButtonClicked(item) {
        // const dialogRef = this.dialog.open(DocumentListByownerComponent, { panelClass: 'modal-dialog' });
        // dialogRef.componentInstance.ownerID = item.rowguid;
        // dialogRef.componentInstance.viewSizeMode = 1; 
    }
};
DataTableComponent.ctorParameters = () => [
    { type: MatDialog },
    { type: Router },
    { type: ActivatedRoute /*, private userDefinedFilterRepo: UserDefinedFilterRepository*/ }
];
__decorate([
    ViewChildren('dataTableFilters')
], DataTableComponent.prototype, "dataTableFilters", void 0);
__decorate([
    ViewChild('pager')
], DataTableComponent.prototype, "pager", void 0);
__decorate([
    ViewChild('tableContainer')
], DataTableComponent.prototype, "tableContainer", void 0);
__decorate([
    ViewChild('actionsContainer')
], DataTableComponent.prototype, "actionsContainer", void 0);
__decorate([
    Input()
], DataTableComponent.prototype, "dataTableID", null);
__decorate([
    Output()
], DataTableComponent.prototype, "onConfigurationLoaded", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onFilterUpdated", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onSortPreferenceChanged", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onGroupPreferenceChanged", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onPageChanged", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onPageCountChanged", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onUserDefinedFilterSelected", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onInsertFormRequested", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onEntitySelected", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onRowSelected", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onColumnSelected", void 0);
__decorate([
    Output()
], DataTableComponent.prototype, "onSelectionChanged", void 0);
__decorate([
    HostListener('document:click', ['$event'])
], DataTableComponent.prototype, "clickedOutside", null);
__decorate([
    HostListener('window:scroll', ['$event'])
], DataTableComponent.prototype, "onWindowScroll", null);
__decorate([
    HostListener('scroll', ['$event'])
], DataTableComponent.prototype, "onScroll", null);
__decorate([
    HostListener('mousewheel', ['$event'])
], DataTableComponent.prototype, "onMouseWheel", null);
DataTableComponent = __decorate([
    Component({
        selector: 'data-table',
        template: "<div class=\"data-table-container\" [attr.identifier]=\"identifier\" style=\"position:relative\">\r\n\r\n  <!--<div class=\"input-icon right\" style=\"float:right;margin:20px 0px\">\r\n  <i class=\"icon-magnifier\"></i>\r\n  <input class=\"form-control form-control-solid input-circle\" placeholder=\"Ara\" [(ngModel)]=\"searchTerm\" (ngModelChange)=\"generalSearchInputChanged($event)\"\r\n  /> </div>-->\r\n  <!-- <div class=\"actions-overlay\" *ngIf=\"selectedAction !== undefined\" (click)=\"toggleActionsDropdown()\"></div> -->\r\n\r\n  <div *ngIf=\"optionsMenuVisible\" class=\"dropdown-menu\"\r\n    style=\"position:absolute; left:auto; right: 30px; top: 30px; display:inline !important;\">\r\n    <div style=\"padding:15px 10px 10px 15px;\">\r\n      <span class=\"pull-left caption-subject font-semi-bold\">\r\n        <ng-template [ngIf]=\"tableSettingMode == 1\">\r\n          Tablo Ayarlar\u0131\r\n        </ng-template>\r\n        <ng-template [ngIf]=\"tableSettingMode == 2\">\r\n          Filtre Y\u00F6netimi\r\n        </ng-template>\r\n      </span>\r\n      <a class=\"fa\" (click)=\"closeOptionsMenu()\" class=\"pull-right\" style=\"font-size:20px\">\r\n        <i class=\"fa fa-remove pull-right\" matTooltip=\"Kapat\" matTooltipPosition=\"above\"></i>\r\n      </a>\r\n      <div class=\"clearfix\"></div>\r\n    </div>\r\n    <ng-template [ngIf]=\"tableSettingMode == 1\">\r\n      <ul class=\"list-group\" style=\"max-height: 300px;margin-bottom: 10px;overflow:scroll;-webkit-overflow-scrolling: touch; border-radius: 0px;overflow-y: scroll;\r\n        overflow-x: hidden;\">\r\n        <li class=\"list-group-item\">\r\n          <!-- <a class=\"pull-left magnifier\" style=\"margin-right:10px\" (click)=\"exportPDF()\">\r\n          <i class=\"fa fa-file-pdf-o\"></i>\r\n        </a>-->\r\n          <a *ngIf=\"isAdmin == true\" class=\"pull-left magnifier\" style=\"margin-right:10px\" (click)=\"editTable()\"\r\n            matTooltip=\"D\u00FCzenle\" matTooltipPosition=\"above\">\r\n            <i class=\"fa fa-pencil\"></i>\r\n          </a>\r\n          <a class=\"pull-left magnifier\" style=\"margin-right:10px\" (click)=\"exportExcel()\" [matTooltip]=\"'Excel indir'\"\r\n            matTooltipPosition=\"above\">\r\n            <i class=\"fa fa-file-excel-o\"></i>\r\n          </a>\r\n          <a class=\"pull-left magnifier\" style=\"margin-right:10px\" (click)=\"toggleSize($event)\"\r\n            [matTooltip]=\"isFullScreen ? 'S\u0131k\u0131\u015Ft\u0131r(Yatay)' : 'Geni\u015Flet(Yatay)'\" matTooltipPosition=\"above\">\r\n            <i [ngClass]=\"isFullScreen ? 'icon-magnifier-remove': 'icon-magnifier-add'\"></i>\r\n          </a>\r\n          <a class=\"pull-left magnifier\" style=\"margin-right:10px\" (click)=\"toggleScroll()\"\r\n            [matTooltip]=\"isScrollable ? 'A\u015Fa\u011F\u0131 Uzat' : 'A\u015Fa\u011F\u0131 Uzatma(Scroll)'\" matTooltipPosition=\"above\">\r\n            <i [ngClass]=\"isScrollable ? 'fa fa-arrows-v': 'fa fa-lock'\"></i>\r\n          </a>\r\n          <a class=\"pull-left magnifier\" style=\"margin-right:10px\" (click)=\"toggleCompactMode()\"\r\n            [matTooltip]=\"isCompressed ? 'Geni\u015Flet(Dikey)' : 'S\u0131k\u0131\u015Ft\u0131r(Dikey)'\" matTooltipPosition=\"above\">\r\n            <i [ngClass]=\"isCompressed ? 'fa fa-compress': 'fa fa-expand'\"></i>\r\n          </a>\r\n          <div class=\"clearfix\"></div>\r\n        </li>\r\n        <ng-template ngFor let-column [ngForOf]=\"columns\" let-i=\"index\">\r\n          <li class=\"list-group-item\">\r\n            <div class=\"pull-left\">\r\n              <app-checkbox [margin]=\"0\" [(checked)]=\"column.isActive\" [label]=\"column.displayText\"\r\n                (checkedChange)=\"activeColumnConfigChanged()\"></app-checkbox>\r\n            </div>\r\n            <div class=\"pull-right\">\r\n              <a *ngIf=\"i > 0\" class=\"fa\" (click)=\"shiftColumn(column, -1)\" class=\"pull-rigt\" style=\"font-size:20px\">\r\n                <i class=\"fa fa-chevron-up pull-right\" style=\"margin-top:10px;\"></i>\r\n              </a>\r\n              <a *ngIf=\"i < columns.length -1\" class=\"fa\" (click)=\"shiftColumn(column, 1)\" class=\"pull-rigt\"\r\n                style=\"font-size:20px\">\r\n                <i class=\"fa fa-chevron-down pull-right\" style=\"margin-top:10px;\"></i>\r\n              </a>\r\n            </div>\r\n            <div class=\"clearfix\"></div>\r\n          </li>\r\n        </ng-template>\r\n      </ul>\r\n    </ng-template>\r\n    <ng-template [ngIf]=\"tableSettingMode == 2\">\r\n      <ul class=\"list-group\"\r\n        style=\"max-height: 300px;margin-bottom: 10px;margin-left:10px;overflow:scroll;-webkit-overflow-scrolling: touch; border-radius: 0px\">\r\n\r\n        <ng-template [ngIf]=\"postInProgress == true\">\r\n          <div class=\"lds-roller\">\r\n            <div></div>\r\n            <div></div>\r\n            <div></div>\r\n            <div></div>\r\n            <div></div>\r\n            <div></div>\r\n            <div></div>\r\n            <div></div>\r\n          </div>\r\n        </ng-template>\r\n\r\n        <ng-template [ngIf]=\"postInProgress == false\">\r\n          <ng-template [ngIf]=\"!userDefinedFilters || userDefinedFilters?.length == 0\">\r\n            <p style=\"margin-top: 10px; margin-bottom: 0 !important\" class=\"text-center font-red\">Herhangi bir kay\u0131t\r\n              bulunamam\u0131\u015Ft\u0131r</p>\r\n          </ng-template>\r\n          <ng-template ngFor let-userDefinedFilter [ngForOf]=\"userDefinedFilters\" let-i=\"index\">\r\n            <ng-template [ngIf]=\"userDefinedFilters?.length > 0\">\r\n              <li class=\"list-group-item\" style=\"display: flex\">\r\n                <ng-template [ngIf]=\"!userDefinedFilter.selected\">\r\n                  {{ userDefinedFilter?.key }}\r\n                </ng-template>\r\n                <div style=\"position: absolute; right: 10px; top: 15px\">\r\n                  <ng-template [ngIf]=\"filterManagementPageMode == 0 && !userDefinedFilter.selected\">\r\n                    <a (click)=\"editUserDefinedFilterActionClicked(userDefinedFilter)\" matTooltip=\"D\u00FCzenle\"\r\n                      matTooltipPosition=\"above\"> <i class=\"fa fa-pencil\"></i> </a>\r\n                  </ng-template>\r\n                  <ng-template [ngIf]=\"filterManagementPageMode == 0 && !userDefinedFilter.selected\">\r\n                    <a (click)=\"deleteUserDefinedFilterActionClicked(userDefinedFilter)\" matTooltip=\"Sil\"\r\n                      matTooltipPosition=\"above\"> <i class=\"fa fa-trash font-red\"></i> </a>\r\n                  </ng-template>\r\n                  <ng-template\r\n                    [ngIf]=\"(filterManagementPageMode == 1 || filterManagementPageMode == 2) && userDefinedFilter.selected == true\">\r\n                    <a (click)=\"saveUserDefinedFilter()\" style=\"margin-top: 7px\"> <i class=\"fa fa-check font-green\"\r\n                        matTooltip=\"Kaydet\" matTooltipPosition=\"above\"></i> </a>\r\n                    <a (click)=\"cancelUserDefinedFilterActionClicked(userDefinedFilter)\" style=\"margin-top: 7px\"\r\n                      matTooltip=\"Vazge\u00E7\" matTooltipPosition=\"above\"> <i class=\"fa fa-times font-red\"></i> </a>\r\n                  </ng-template>\r\n                  <ng-template\r\n                    [ngIf]=\"filterManagementPageMode == 0 && !userDefinedFilter.selected && userDefinedFilter.activated == false\">\r\n                    <a (click)=\"activateActionClicked(userDefinedFilter)\" style=\"margin-left: 5px\">Etkinle\u015Ftir</a>\r\n                  </ng-template>\r\n                  <ng-template\r\n                    [ngIf]=\"filterManagementPageMode == 0 && !userDefinedFilter.selected && userDefinedFilter.activated == true\">\r\n                    <span class=\"label label-success\" style=\"margin-left: 5px\">Etkin</span>\r\n                  </ng-template>\r\n                </div>\r\n                <ng-template\r\n                  [ngIf]=\"(filterManagementPageMode == 1 || filterManagementPageMode == 2) && userDefinedFilter.selected == true\">\r\n                  <input type=\"text\" class=\"form-control\" [(ngModel)]=\"activeUserDefinedFilter.name\"\r\n                    placeholder=\"Filtre Ad\u0131\" style=\"width: 75%\">\r\n                </ng-template>\r\n              </li>\r\n            </ng-template>\r\n          </ng-template>\r\n        </ng-template>\r\n      </ul>\r\n    </ng-template>\r\n    <!-- <ng-template [ngIf]=\"onUserDefinedFilterSelected.observers.length > 0\"> -->\r\n\r\n    <ng-template [ngIf]=\"tableSettingMode == 1\">\r\n      <div class=\"form-group\">\r\n        <button class=\"btn btn-default btn-sm btn-block pull-right\" (click)=\"switchTableSettingMode()\">\r\n          Filtre Y\u00F6netimi\r\n        </button>\r\n      </div>\r\n    </ng-template>\r\n\r\n    <div class=\"form-inline\" style=\"margin: 10px\">\r\n      <ng-template [ngIf]=\"tableSettingMode == 2\">\r\n        <div class=\"form-group\">\r\n          <button class=\"btn btn-default btn-sm btn-block pull-right\" (click)=\"switchTableSettingMode()\">\r\n            <i class=\"fa fa-chevron-left\"></i> Geri D\u00F6n\r\n          </button>\r\n        </div>\r\n      </ng-template>\r\n\r\n      <ng-template [ngIf]=\"tableSettingMode == 2 && filterManagementPageMode == 0\">\r\n        <div class=\"form-group\">\r\n          <button class=\"btn btn-success btn-sm btn-block pull-right\" (click)=\"turnDefaultFilterActionClicked()\">\r\n            Varsay\u0131lana D\u00F6n\r\n          </button>\r\n        </div>\r\n        <div class=\"form-group\">\r\n          <button class=\"btn btn-primary btn-sm btn-block pull-right\" (click)=\"addNewFilterButtonClicked()\">\r\n            Yeni Ekle\r\n          </button>\r\n        </div>\r\n\r\n      </ng-template>\r\n    </div>\r\n    <!-- </ng-template> -->\r\n  </div>\r\n\r\n  <div class=\"table-container\" [class.hidden]=\"loading || !enabled\" [class.has-actions]=\"columnActions\">\r\n    <div class=\"table-column\">\r\n      <!-- <ng-scrollbar [trackY]=\"false\" [trackX]=\"true\" [autoHide]=\"true\" thumbClass=\"bar-bottom\"> -->\r\n      <div style=\"margin-top:0 !important\" class=\"table-responsive dafaque\" [class.table-scroll-enabled]=\"isScrollable\"\r\n        [class.table-scroll-disabled]=\"!isScrollable\" [class.table-nowrap]=\"isCompressed\" #tableContainer>\r\n        <div *ngFor=\"let column of activeColumns\">\r\n          <button *ngIf=\"column.grouped\">{{column.displayText}}</button>\r\n        </div>\r\n        <table class=\"table application-data-table\">\r\n          <thead>\r\n            <tr>\r\n              <th *ngIf=\"uiSelectionEnabled\"></th>\r\n              <th *ngFor=\"let column of activeColumns\">\r\n                <div>\r\n                  <h4>{{ column.displayText }}</h4>\r\n                  <a *ngIf=\"column.supportsSorting\" class=\"margin-l-5 pull-left padding-t-10\" (click)=\"sort(column)\">\r\n                    <i *ngIf=\"column.sortDirection == 0\" class=\"fa fa-sort-desc\"></i>\r\n                    <i *ngIf=\"column.sortDirection == 1\" class=\"fa fa-sort-asc\"></i>\r\n                    <i *ngIf=\"column.sortDirection == 2\" class=\"fa fa-sort\"></i>\r\n                  </a>\r\n                  <a *ngIf=\"column.supportsGrouping\" class=\"margin-r-5 pull-right font-dark padding-t-10\"\r\n                    (click)=\"group(column)\">\r\n                    <i class=\"fa fa-link\"></i>\r\n                  </a>\r\n                </div>\r\n              </th>\r\n              <th *ngIf=\"commentsEnabled\"></th>\r\n              <th *ngIf=\"documentsEnabled\"></th>\r\n              <th class=\"sticky-column-with-fake-border\" style=\"text-align:center;min-width: 50px;\">\r\n                <i (click)=\"toggleOptionsMenu()\" class=\"pe-7s-keypad\" aria-hidden=\"true\"\r\n                  style=\"font-size:24px;color:#4a4a4a; margin-top:8px; cursor:pointer;\" matTooltip=\"Ayarlar\u0131 uygula\"\r\n                  matTooltipPosition=\"above\"></i>\r\n                <!-- <h4 class=\"hidden-xs\"> {{actionColumn.displayText}} </h4> -->\r\n                <!-- <h4 class=\"visible-xs\"> {{actionColumn.displayText | truncate: 4}} </h4> -->\r\n              </th>\r\n              <th></th>\r\n            </tr>\r\n          </thead>\r\n          <tbody>\r\n            <tr *ngIf=\"columnFilterEnabled\" class=\"filter-row\">\r\n              <td *ngIf=\"uiSelectionEnabled\">\r\n                <app-checkbox [(checked)]=\"allSelected\" (checkedChange)=\"toggleAllSelection()\"></app-checkbox>\r\n              </td>\r\n              <td *ngFor=\"let column of activeColumns\">\r\n                <data-table-filter #dataTableFilters *ngIf=\"column.filterEnabled == true\" [(filter)]=\"column.filter\"\r\n                  (onFilterChanged)=\"onFilterChanged()\" (comboboxItemsLoaded)=\"changeFilterState()\">\r\n                </data-table-filter>\r\n                <ng-template [ngIf]=\"column.filterEnabled == false\">\r\n                  <!-- {{ 'MESSAGES.filteringDisabled' | translate }} -->\r\n                </ng-template>\r\n              </td>\r\n              <td *ngIf=\"commentsEnabled\"></td>\r\n              <td *ngIf=\"documentsEnabled\"></td>\r\n              <td class=\"sticky-column-with-fake-border\">\r\n                <div>\r\n                  <a matTooltip=\"Filtreleri Temizle\" matTooltipPosition=\"above\" class=\"clear-filters\"\r\n                    [ngClass]=\"{ 'hidden': !hasFilters }\" (click)=\"clearFilters()\">\r\n                    <i class=\"iconx-remove-filter font-red-haze\"></i>\r\n                  </a>\r\n                </div>\r\n              </td>\r\n            </tr>\r\n            <ng-template [ngIf]=\"containsNestedData == true\">\r\n              <ng-template ngFor let-item [ngForOf]=\"items\" let-rowIndex=\"index\">\r\n                <tr *ngFor=\"let row of item._rowIterator; let i=index\" [ngStyle]=\"item?._render?._deletedCondition\"\r\n                  (dblclick)=\"itemClicked(item)\" (contextmenu)=\"onRightClick($event, item)\">\r\n                  <td [rowSpan]=\"item._rowCount\" *ngIf=\"item._render._uiSelectionEnabled[i]\">\r\n                    <app-checkbox [(checked)]=\"item._selected\" (checkedChange)=\"checkedChanged(item)\"></app-checkbox>\r\n                  </td>\r\n                  <ng-template ngFor let-column [ngForOf]=\"activeColumns\" let-columnIndex=\"index\">\r\n                    <ng-template [ngIf]=\"column.nestedDataStrategy == 2\">\r\n                      <td [rowSpan]=\"item._render[column.field][i]['rowSpan']\"\r\n                        *ngIf=\"item._render[column.field][i]['rowSpanEnabled']\"\r\n                        (click)=\"columnSelected(rowIndex, columnIndex, item)\"\r\n                        [ngStyle]=\"{'min-width':  column.minWidth ? column.minWidth: '75px','background-color':  ((item == activeItem) && (columnIndex == activeColumn)) ? '#d3dbe9': 'transparent', 'border': ((rowIndex == rowToBeSelected) && (columnIndex == columnToBeSelected)) ? '1px dashed black': '1px solid #e7ecf1'}\">\r\n                        <ng-template [ngIf]=\"column.type == 'condition'\">\r\n                          <div [innerHTML]=\"item._render['_template_' + column.index]\"></div>\r\n                        </ng-template>\r\n                        <ng-template [ngIf]=\"column.type == 'html'\">\r\n                          <div [innerHTML]=\"item._render[column.field]\"></div>\r\n                        </ng-template>\r\n                        <ng-template [ngIf]=\"column.type != 'condition'\">\r\n                          {{item._render[column.field][i]['displayText']}}</ng-template>\r\n\r\n                        <ng-template [ngIf]=\"column.type == 'multimedia'\">\r\n                          <img src=\"{{baseURL}}multimedia/{{item._render[column.field]}}\" class=\"media-object\"\r\n                            width=\"200px;\" />\r\n                        </ng-template>\r\n\r\n                        <ng-template [ngIf]=\"column.type == 'video'\">\r\n                          <video width=\"200\" height=\"200\" controls>\r\n                            <source src=\"{{baseURL}}multimedia/{{item._render[column.field]}}\" type=\"video/mp4\">\r\n                            Your browser does not support the video tag.\r\n                          </video>\r\n                        </ng-template>\r\n                      </td>\r\n                    </ng-template>\r\n                    <ng-template [ngIf]=\"column.nestedDataStrategy != 2\">\r\n                      <td [rowSpan]=\"item._rowCount\" *ngIf=\"item._render._tdVisible[i]\"\r\n                        (click)=\"columnSelected(rowIndex, columnIndex, item)\"\r\n                        [ngStyle]=\"{'min-width':  column.minWidth ? column.minWidth: '75px','background-color':  ((rowIndex == activeRow) && (columnIndex == activeColumn)) ? '#d3dbe9': 'transparent', 'border': ((rowIndex == rowToBeSelected) && (columnIndex == columnToBeSelected)) ? '1px dashed black': '1px solid #e7ecf1'}\">\r\n                        <ng-template [ngIf]=\"column.type == 'condition'\">\r\n                          <div [innerHTML]=\"item._render['_template_' + column.index]\"></div>\r\n                        </ng-template>\r\n                        <ng-template [ngIf]=\"column.type == 'html'\">\r\n                          <div [innerHTML]=\"item._render[column.field]\"></div>\r\n                        </ng-template>\r\n                        <ng-template [ngIf]=\"column.type == 'data' || column.type == 'date'\">\r\n                          {{item._render[column.field]}}\r\n                        </ng-template>\r\n\r\n                        <ng-template [ngIf]=\"column.type == 'multimedia'\">\r\n                          <img src=\"{{baseURL}}multimedia/{{item._render[column.field]}}\" class=\"media-object\"\r\n                            width=\"200px;\" />\r\n                        </ng-template>\r\n\r\n                        <ng-template [ngIf]=\"column.type == 'video'\">\r\n                          <video width=\"200\" height=\"200\" controls>\r\n                            <source src=\"{{baseURL}}multimedia/{{item._render[column.field]}}\" type=\"video/mp4\">\r\n                            Your browser does not support the video tag.\r\n                          </video>\r\n                        </ng-template>\r\n                      </td>\r\n                    </ng-template>\r\n                  </ng-template>\r\n                  <td [rowSpan]=\"item._rowCount\" *ngIf=\"item._render._tdVisible[i] && commentsEnabled\"\r\n                    class=\"dt-action-column\" (click)=\"showCommentsButtonClicked(item)\" matTooltip=\"Yorumlar\"\r\n                    matTooltipPosition=\"above\">\r\n                    <i class=\"fa fa-comments\"></i>\r\n                    <span class=\"badge badge-danger\" *ngIf=\"item.commentCount > 0\">{{item.commentCount}}</span>\r\n                  </td>\r\n                  <td [rowSpan]=\"item._rowCount\" *ngIf=\"item._render._tdVisible[i] && documentsEnabled\"\r\n                    class=\"dt-action-column\" (click)=\"showDocumentsButtonClicked(item)\" matTooltip=\"Dok\u00FCmanlar\"\r\n                    matTooltipPosition=\"above\">\r\n                    <i class=\"fa fa-file-text-o\"></i>\r\n                    <span class=\"badge badge-danger\" *ngIf=\"item.documentCount > 0\">{{item.documentCount}}</span>\r\n                  </td>\r\n                  <td [rowSpan]=\"item._rowCount\" class=\"sticky-column-with-fake-border\"\r\n                    *ngIf=\"item._render._tdVisible[i]\" style=\"text-align: center\">\r\n                    <a class=\"dropdown-toggle row-actions-btn\" (click)=\"toggleActionsDropdown($event, item)\"\r\n                      data-toggle=\"dropdown\" aria-expanded=\"true\"><i class=\"pe-7s-more\" aria-hidden=\"true\"\r\n                        style=\"color:#d3dbe9 !important;font-size: 18px;\"></i></a>\r\n                  </td>\r\n                </tr>\r\n              </ng-template>\r\n            </ng-template>\r\n            <ng-template [ngIf]=\"containsNestedData == false\">\r\n              <tr *ngFor=\"let item of items; let rowIndex = index\" [ngStyle]=\"item?._render?._deletedCondition\"\r\n                class=\"data-row bordered-row\" (dblclick)=\"itemClicked(item)\" (contextmenu)=\"onRightClick($event, item)\"\r\n                [ngClass]=\"item._disabled ? 'invalid-row' : ''\">\r\n                <td *ngIf=\"uiSelectionEnabled\">\r\n                  <app-checkbox [(checked)]=\"item._selected\" (checkedChange)=\"checkedChanged(item)\"></app-checkbox>\r\n                </td>\r\n                <td *ngFor=\"let column of activeColumns; let columnIndex = index\" [title]=\"item._render[column.field]\"\r\n                  (click)=\"columnSelected(rowIndex, columnIndex, item)\"\r\n                  [ngStyle]=\"{'min-width':  column.minWidth ? column.minWidth: '75px','background-color':  ((rowIndex == activeRow) && (columnIndex == activeColumn)) ? '#d3dbe9': 'transparent', 'border': ((rowIndex == rowToBeSelected) && (columnIndex == columnToBeSelected)) ? '1px dashed black': '1px solid #e7ecf1'}\">\r\n                  <ng-template [ngIf]=\"column.type == 'condition'\">\r\n                    <div [innerHTML]=\"item._render['_template_' + column.index]\"></div>\r\n                  </ng-template>\r\n                  <ng-template [ngIf]=\"column.type == 'html'\">\r\n                    <div [innerHTML]=\"item._render[column.field]\"></div>\r\n                  </ng-template>\r\n                  <ng-template [ngIf]=\"column.type == 'data' || column.type == 'date'\">\r\n                    {{item._render[column.field]}}\r\n                  </ng-template>\r\n\r\n                  <ng-template [ngIf]=\"column.type == 'multimedia'\">\r\n                    <img src=\"{{baseURL}}multimedia/{{item._render[column.field]}}\" class=\"media-object\"\r\n                      width=\"200px;\" />\r\n                  </ng-template>\r\n\r\n                  <ng-template [ngIf]=\"column.type == 'video'\">\r\n                    <video width=\"200\" height=\"200\" controls>\r\n                      <source src=\"{{baseURL}}multimedia/{{item._render[column.field]}}\" type=\"video/mp4\">\r\n                      Your browser does not support the video tag.\r\n                    </video>\r\n                  </ng-template>\r\n                </td>\r\n                <td *ngIf=\"commentsEnabled\" class=\"dt-action-column\" (click)=\"showCommentsButtonClicked(item)\"\r\n                  matTooltip=\"Yorumlar\" matTooltipPosition=\"above\">\r\n                  <i class=\"fa fa-comments\"></i>\r\n                  <span class=\"badge badge-danger\" *ngIf=\"item.commentCount > 0\">{{item.commentCount}}</span>\r\n                </td>\r\n                <td *ngIf=\"documentsEnabled\" class=\"dt-action-column\" (click)=\"showDocumentsButtonClicked(item)\"\r\n                  matTooltip=\"Dok\u00FCmanlar\" matTooltipPosition=\"above\">\r\n                  <i class=\"fa fa-file-text-o\"></i>\r\n                  <span class=\"badge badge-danger\" *ngIf=\"item.documentCount > 0\">{{item.documentCount}}</span>\r\n                </td>\r\n                <td class=\"sticky-column-with-fake-border\" style=\"text-align: center\">\r\n                  <a class=\"dropdown-toggle row-actions-btn\" (click)=\"toggleActionsDropdown($event, item)\"\r\n                    data-toggle=\"dropdown\" aria-expanded=\"true\"><i class=\"pe-7s-more\" aria-hidden=\"true\" style=\"color:#d3dbe9 !important;font-size: 18px;vertical-align: middle;\r\n                      \"></i></a>\r\n                </td>\r\n              </tr>\r\n            </ng-template>\r\n            <tr *ngIf=\"placeboRowHeight > 0\" [style.height.px]=\"placeboRowHeight\">\r\n              <td [colSpan]=\"columnCount + 1\">\r\n                <div [class.hidden]=\"loading || !enabled || items.length > 0\">\r\n                  <ng-template [ngIf]=\"emptyResultPagerTemplate === undefined\">\r\n                    <p\r\n                      style=\"text-align:center; width: 1100px !important /* Switch to css with media queries to support responsive */;\">\r\n                      {{'LISTS.preEntityNotFound' | translate}}\r\n                      <!-- <a (click)=\"presentInsertForm()\"> {{'LISTS.entityNotFoundNavigation' | translate}} </a> {{'LISTS.postEntityNotFound' | translate}} -->\r\n                    </p>\r\n                  </ng-template>\r\n                  <ng-template [ngIf]=\"emptyResultPagerTemplate !== undefined\">\r\n                    <div [innerHTML]=\"emptyResultPagerTemplate\"></div>\r\n                  </ng-template>\r\n                </div>\r\n              </td>\r\n            </tr>\r\n          </tbody>\r\n        </table>\r\n      </div>\r\n      <!-- </ng-scrollbar> -->\r\n    </div>\r\n  </div>\r\n  <div class=\"row\">\r\n    <div class=\"col-md-4 col-sm-4\">\r\n      <div *ngIf=\"pageCount >= itemCount && itemCount > 0\" class=\"dataTables_info\" translate=\"MESSAGES.allRecords\"\r\n        [translateParams]=\"{itemCount:itemCount}\"></div>\r\n      <div *ngIf=\"pageCount < itemCount\" class=\"dataTables_info\" translate=\"MESSAGES.partialRecords\"\r\n        [translateParams]=\"{itemCount:itemCount, startIndex:(pageIndex*pageCount + 1), endIndex:((pageIndex*pageCount + pageCount) > itemCount ? itemCount : pageIndex*pageCount + pageCount)}\">\r\n      </div>\r\n    </div>\r\n    <div class=\"col-md-6 col-sm-6\">\r\n      <ng-template [ngIf]=\"itemCount > 0\">\r\n        <div class=\"dataTables_paginate paging_bootstrap_number\" id=\"sample_3_paginate\">\r\n          <pager #pager (onPageIndexChange)=\"pageChanged($event)\" [(itemCount)]=\"itemCount\"\r\n            [(activePageIndex)]=\"pageIndex\" [(itemCountPerPage)]=\"pageCount\"> </pager>\r\n        </div>\r\n      </ng-template>\r\n    </div>\r\n    <div class=\"col-md-2 col-sm-2\">\r\n      <select *ngIf=\"items != undefined && itemCount * 2 > 50\" class=\"form-control\" id=\"pageCount\" name=\"pageCount\"\r\n        [(ngModel)]=\"pageCount\" (ngModelChange)=\"pageCountChanged($event)\">\r\n        <option *ngIf=\"itemCount * 2 > 50\" value=50>50</option>\r\n        <option *ngIf=\"itemCount * 2 > 250\" value=250>250</option>\r\n        <option *ngIf=\"itemCount * 2 > 500\" value=500>500</option>\r\n        <option *ngIf=\"itemCount * 2 > 501\" value=0>T\u00FCm\u00FC</option>\r\n      </select>\r\n    </div>\r\n  </div>\r\n\r\n\r\n  <div [class.hidden]=\"loading || !enabled || items.length > 0\" *ngIf=\"emptyResultTemplateShownInPager == true\">\r\n    <!-- <ng-template [ngIf]=\"emptyResultPagerTemplate === undefined\" [ngIfElse]=\"emptyResultTemplate\">\r\n    {{'LISTS.preEntityNotFound' | translate}} <a (click)=\"presentInsertForm()\"> {{'LISTS.entityNotFoundNavigation' | translate}} </a> {{'LISTS.postEntityNotFound' | translate}}\r\n  </ng-template> -->\r\n    <ng-template #emptyResultTemplate>\r\n      <div [innerHTML]=\"emptyResultPagerTemplate\"></div>\r\n    </ng-template>\r\n  </div>\r\n  <app-loader *ngIf=\"loading\"></app-loader>\r\n\r\n  <div [class.hidden]=\"activeContextItem === undefined\" [attr.id]=\"'action_menu_container_' + identifier\">\r\n    <ul class=\"dropdown-menu pull-right\" role=\"menu\" style=\"right: 49px; top: 19px;\"\r\n      [attr.id]=\"'action_menu_content_' + identifier\">\r\n      <ng-template ngFor let-action [ngForOf]=\"columnActions\">\r\n        <ng-template [ngIf]=\"activeContextItem !== undefined\">\r\n          <li *ngIf=\"action.meetsConditions(activeContextItem)\">\r\n            <a (click)=\"actionTriggered(action,activeContextItem)\"><i class=\"{{action.icon}}\"></i>\r\n              <ng-template [ngIf]=\"!action.needsTranslation\" [ngIfElse]=\"translationTemplate\">\r\n                {{action.displayText}}\r\n              </ng-template>\r\n              <ng-template #translationTemplate>\r\n                {{action.displayText | translate}}\r\n              </ng-template>\r\n            </a>\r\n          </li>\r\n        </ng-template>\r\n      </ng-template>\r\n    </ul>\r\n  </div>\r\n</div>",
        encapsulation: ViewEncapsulation.None,
        host: { '(window:keydown)': 'keyboardInput($event)' },
        providers: [ /*UserDefinedFilterRepository*/]
        //changeDetection: ChangeDetectionStrategy.OnPush
        ,
        styles: [".actions-overlay{content:\" \";position:fixed;background-color:rgba(255,255,255,.1);width:100%;height:100%;z-index:3;left:0;top:0}.dataTables_wrapper{background:0 0}.table-scrollable{border-radius:5px;background-color:#fff;box-shadow:0 2px 3px 0 rgba(0,0,0,.24);border:none!important;overflow-y:visible!important}.application-data-table thead{width:1144.6px;height:50px;background-color:#d3dbe9}.application-data-table thead tr td h4{font-family:\"Open Sans\",sans-serif;letter-spacing:.5px;text-align:left;font-weight:500;color:#4a4a4a;padding-left:4px;padding-right:4px}.application-data-table thead tr th h4{font-family:\"Open Sans\",sans-serif;letter-spacing:.5px;text-align:left;font-weight:500;color:#4a4a4a;padding-left:4px;padding-right:4px;font-size:14px}.application-data-table thead tr th>div{display:flex;justify-content:space-between}.application-data-table tbody tr td .btn-group{position:absolute}.actions-column .dropdown-menu{margin-top:10px!important}.actions-column .dropdown-menu li{background-color:#fafcff}.actions-column .dropdown-menu li a{transition:.3s cubic-bezier(.175,.885,.32,1.275)}.actions-column .dropdown-menu li:not(:first-child){margin:3px 0}.actions-column .dropdown-menu li a:hover{background-color:rgba(55,130,195,.2)}.actions-column .btn-group.open .dropdown-menu li:first-child:before{content:\" \";right:5px;z-index:999;position:absolute;width:0;height:0;top:-9px;border:5px solid transparent;border-top:5px solid rgba(55,130,195,.2)}.row-actions-btn{border-radius:50%;width:35px;height:25px;display:inline-block;padding-top:2px;margin:0 auto}.row-actions-btn img{max-height:20px;margin:0 2px 2px}@media (min-width:736px) and (max-width:1024px){.btn-group.open .dropdown-menu li:first-child:before{content:\" \";right:5px;z-index:999;position:absolute;width:0;height:0;top:-9px;border:5px solid transparent;border-top:5px solid #3782c3}}@media (max-width:544px){.table-scrollable{position:relative}.table-scrollable .row-actions-column .btn-group{right:48px}.table-responsive .dropdown-menu{position:fixed;bottom:0;width:100%;max-height:100%;top:auto;overflow:scroll;padding:0;z-index:1000;margin:0;min-height:92px;transition:.5s cubic-bezier(.175,.885,.32,1.275)}.table-responsive .input-group-btn.open:before{display:block;width:100%;height:100%;content:\" \";top:0;left:0;background-color:rgba(217,230,240,.5);z-index:9;position:fixed}.table-scrollable .dropdown-menu li{position:relative!important;display:block!important;z-index:1200;padding:3px;margin:5px;border-radius:8px}.dropdown-backdrop{background-color:rgba(206,221,239,.2);z-index:1}.application-data-table tbody .row-actions-column .btn-group{right:48px}.row-actions-column{position:relative}}.row-actions-column .btn.btn-circle{width:25px;height:25px;padding:2.5px 0;font-size:12px}.table-container{margin-bottom:20px}.table-container table{margin-bottom:5px;min-height:150px;background-color:#fff}.table-container table tr.mock-row{height:150px}.table-container table thead th{white-space:nowrap;background-color:#d3dbe9!important;color:#4a4a4a}.table-container table thead th h4{font-family:\"Open Sans\",sans-serif;letter-spacing:.5px;text-align:left;font-weight:500;color:#4a4a4a;padding-left:4px;padding-right:4px;font-size:14px}.table-container .table-responsive{border:none;box-shadow:0 2px 3px 0 rgba(0,0,0,.24)}.table-container:before{display:table;content:\" \"}.table-container:after{display:table;content:\" \";clear:both}.table-container>div{padding:0!important;float:left;position:relative;z-index:1}.table-container .table-column th>div{display:flex;justify-content:space-between}.table-container:not(.has-actions){border-radius:5px}.table-container:not(.has-actions) .table-column{width:100%}.table-container:not(.has-actions) .table-column .table-responsive{border-radius:5px}.table-container.has-actions .table-column{width:calc(100%)}.table-container.has-actions .table-column .table-responsive{border-top-left-radius:5px;border-bottom-right-radius:5px}.table-container.has-actions .table-column .table-responsive table tbody .filter-row{height:42px}.table-container.has-actions .table-column .table-responsive table tbody .filter-row td{background-color:#fff;position:-webkit-sticky;position:sticky;top:38px;z-index:1000;padding:4px!important;vertical-align:middle!important}.table-container.has-actions .table-column .table-responsive table tbody .filter-row boolean-filter .form-group{margin:-1px 4px 4px!important}.table-container.has-actions .table-column .table-responsive table tbody .filter-row .dropdown button,.table-container.has-actions .table-column .table-responsive table tbody .filter-row input-filter .input-group{margin-top:-5px}.table-container.has-actions .actions-column{width:54px;border-left:3px solid #d3dbe9;text-align:center;z-index:4;max-height:480px;overflow-y:auto;padding-bottom:18px!important}.table-container.has-actions .actions-column th{border-top-right-radius:5px}.table-container.has-actions .actions-column th>div{min-height:35px;text-align:center;padding-top:5px}.table-container.has-actions .actions-column th h4{text-align:center}.table-container.has-actions .actions-column td{padding:5px;white-space:nowrap;vertical-align:middle}.table-container.has-actions .actions-column td .btn-group.open>ul{position:fixed;z-index:1;top:464px;right:425px;height:auto}@media (max-width:644px){.table-container.has-actions .actions-column .row-actions-column .btn-group button i{margin:0 0 0 -1px!important}.table-container.has-actions .actions-column .row-actions-column .btn-group .dropdown-menu li:first-child:before{right:.5em;top:-11px}.table-container.has-actions .actions-column .row-actions-column .dropdown-menu{position:fixed;bottom:0;width:100%;max-height:100%;top:auto;overflow:scroll;padding:0;z-index:1000;margin:0;min-height:92px;transition:.5s cubic-bezier(.175,.885,.32,1.275)}.table-container.has-actions .actions-column .row-actions-column .dropdown-menu li{position:relative!important;display:block!important;z-index:1200;padding:3px;margin:5px;border-radius:8px}.table-container.has-actions .actions-column .btn-group.open:before,.table-container.has-actions .actions-column .dropdown.open{display:block;width:100%;height:100%;content:\" \";top:0;left:0;background-color:rgba(217,230,240,.5);z-index:9;position:fixed}}.dropdown-backdrop{z-index:1}::-webkit-scrollbar{width:10px;background-color:#fff}::-webkit-scrollbar-track{border-radius:3px;background-color:#fff}::-webkit-scrollbar-thumb{border-radius:3px;background-color:#d3dbe9}.application-data-table thead tr th{position:-webkit-sticky;position:sticky;top:0;z-index:1000}.sticky-column-with-fake-border{background:url(data:image/gif;base64,R0lGODlhAQABAIAAAMLCwgAAACH5BAAAAAAALAAAAAABAAEAAAICRAEAOw==) left repeat-y #fff;width:50px!important;position:-webkit-sticky;position:sticky;right:0;z-index:50}.magnifier{font-size:1.5em}.clear-filters{font-size:1.2em}.application-data-table>tbody>tr>td,.application-data-table>tbody>tr>th,.application-data-table>tfoot>tr>td,.application-data-table>tfoot>tr>th,.application-data-table>thead>tr>td,.application-data-table>thead>tr>th{vertical-align:middle!important;padding:2px}.dt-action-column{padding:10px!important;font-size:20px!important;color:#adadad!important;cursor:pointer}.dt-action-column .badge{margin-top:-25px;margin-left:-13px}.table-nowrap table tbody tr{white-space:nowrap}.table-nowrap table tbody tr td{margin:0!important;padding:0 0 0 5px!important}.table-nowrap table tbody tr td app-checkbox div{margin:0 -2px 0 2px!important;padding:0!important}.table-nowrap table tbody tr td app-checkbox div .application-checkbox-selected{margin:2px -2px 0 2px!important}.table-scroll-enabled{max-height:480px;overflow-y:scroll}.table-scroll-disabled{max-height:unset;overflow-y:visible}.table-nowrap .data-row .input-group-addon,.table-nowrap .data-row input,.table-nowrap .data-row select{border:none!important;background-color:transparent!important}.table-nowrap .data-row .input-group-addon{background-color:transparent!important;border-radius:0;border-left:1px solid #e3e3e3!important;padding-left:5px!important;padding-right:0!important}.invalid-row{background-color:#f7c9c5!important}.bordered-row:hover{box-shadow:inset 1px 0 0 #dadce0,inset -1px 0 0 #dadce0,0 1px 2px 0 rgba(60,64,67,.3),0 1px 3px 1px rgba(60,64,67,.15)}.sticky-column-with-fake-border a:hover{text-decoration:none!important;background-color:#f3f6f8;transition:background-color .7s cubic-bezier(.71,.29,.21,.49)}.lds-roller{display:inline-block;position:relative;width:64px;height:64px;left:50%;transform:translate(-50%)}.lds-roller div{-webkit-animation:1.2s cubic-bezier(.5,0,.5,1) infinite lds-roller;animation:1.2s cubic-bezier(.5,0,.5,1) infinite lds-roller;transform-origin:32px 32px}.lds-roller div:after{content:\" \";display:block;position:absolute;width:6px;height:6px;border-radius:50%;background:#000;margin:-3px 0 0 -3px}.lds-roller div:nth-child(1){-webkit-animation-delay:-36ms;animation-delay:-36ms}.lds-roller div:nth-child(1):after{top:50px;left:50px}.lds-roller div:nth-child(2){-webkit-animation-delay:-72ms;animation-delay:-72ms}.lds-roller div:nth-child(2):after{top:54px;left:45px}.lds-roller div:nth-child(3){-webkit-animation-delay:-108ms;animation-delay:-108ms}.lds-roller div:nth-child(3):after{top:57px;left:39px}.lds-roller div:nth-child(4){-webkit-animation-delay:-144ms;animation-delay:-144ms}.lds-roller div:nth-child(4):after{top:58px;left:32px}.lds-roller div:nth-child(5){-webkit-animation-delay:-.18s;animation-delay:-.18s}.lds-roller div:nth-child(5):after{top:57px;left:25px}.lds-roller div:nth-child(6){-webkit-animation-delay:-216ms;animation-delay:-216ms}.lds-roller div:nth-child(6):after{top:54px;left:19px}.lds-roller div:nth-child(7){-webkit-animation-delay:-252ms;animation-delay:-252ms}.lds-roller div:nth-child(7):after{top:50px;left:14px}.lds-roller div:nth-child(8){-webkit-animation-delay:-288ms;animation-delay:-288ms}.lds-roller div:nth-child(8):after{top:45px;left:10px}@-webkit-keyframes lds-roller{0%{transform:rotate(0)}100%{transform:rotate(360deg)}}@keyframes lds-roller{0%{transform:rotate(0)}100%{transform:rotate(360deg)}}"]
    })
], DataTableComponent);
var TableSettingMode;
(function (TableSettingMode) {
    TableSettingMode[TableSettingMode["Default"] = 1] = "Default";
    TableSettingMode[TableSettingMode["FilterManagement"] = 2] = "FilterManagement";
})(TableSettingMode || (TableSettingMode = {}));
var FilterManagementPageMode;
(function (FilterManagementPageMode) {
    FilterManagementPageMode[FilterManagementPageMode["List"] = 0] = "List";
    FilterManagementPageMode[FilterManagementPageMode["Insert"] = 1] = "Insert";
    FilterManagementPageMode[FilterManagementPageMode["Update"] = 2] = "Update";
})(FilterManagementPageMode || (FilterManagementPageMode = {}));
class UserDefinedFilterModel {
}

class Action {
    constructor() {
        this.conditionDirection = true;
        this.needsTranslation = false;
    }
    meetsConditions(item) {
        if (this.conditionField == undefined && this.conditions == undefined)
            return true;
        if (this.conditionField != undefined) {
            if ((this.conditionDirection == true && this.condition == item[this.conditionField]) || (this.conditionDirection == false && this.condition != item[this.conditionField])) {
                return true;
            }
            else {
                return false;
            }
        }
        else if (this.conditions != undefined) {
            for (const condition of this.conditions) {
                if (condition.field != undefined) {
                    if (!((condition.direction == true && condition.condition == item[condition.field]) || (condition.direction == false && condition.condition != item[condition.field]))) {
                        return false;
                    }
                }
            }
        }
        return true;
    }
    addCondition(field, condition, direction = true) {
        if (this.conditions == undefined)
            this.conditions = new Array();
        var item = new Condition();
        item.condition = condition;
        item.field = field;
        item.direction = direction;
        this.conditions.push(item);
    }
    static parse(element) {
        if (element == undefined)
            return undefined;
        var action = new Action();
        action.displayText = element.displayText;
        action.actionMethodName = element.actionMethodName;
        action.action = element.action;
        action.icon = element.icon;
        action.hasCondition = element.hasCondition;
        action.condition = element.condition;
        action.conditionField = element.conditionField;
        if (action.displayText.startsWith("FORMS.")) {
            action.needsTranslation = true;
        }
        return action;
    }
}

class TableConfiguration {
    constructor() {
        this.enabled = true;
        this.actionHeader = "";
    }
    static parseList(objects) {
        if (objects == undefined)
            return undefined;
        let items = new Array();
        objects.forEach(element => {
            items.push(TableConfiguration.parse(element));
        });
        return items;
    }
    static parse(element) {
        if (element == undefined)
            return undefined;
        var configuration = new TableConfiguration();
        configuration.columns = Column.parseList(element["columns"]);
        var actionColumn = element["actionColumn"];
        if (actionColumn !== undefined) {
            configuration.actionColumn = new Column();
            configuration.actionColumn.displayText = actionColumn["displayText"];
            configuration.actionHeader = actionColumn["actionButtonDisplayText"];
        }
        configuration.enabled = element["enabled"];
        return configuration;
    }
}

let GenericFilterRepository = class GenericFilterRepository {
    constructor(http) {
        this.http = http;
    }
    getAll(target, pageIndex, pageCount, includeInactive, filter = undefined, sort = undefined, extendStrings = undefined, includeDeleted = false, includeSystemColumns = undefined) {
        var baseRepository = new BaseRepository(undefined);
        var data = { pageIndex: pageIndex, pageCount: pageCount, includeInactive: includeInactive, includeSystemColumns: includeSystemColumns };
        baseRepository.applyCriteria(data, filter, sort, extendStrings);
        var deletedPostFix = includeDeleted === true ? '/deleted' : '';
        return this.http
            .post(baseRepository.apiRoot + target + "/list" + deletedPostFix, JSON.stringify(data), { headers: baseRepository.defaultHeaders() }).toPromise()
            .then(responseObject => {
            return new ResponseModel(undefined, responseObject);
        });
    }
    static getPath(input) {
        var path = input.replace(/([a-z])([A-Z])/g, '$1-$2');
        path = this.pluralize(path);
        return path;
    }
    static pluralize(element) {
        var path = element;
        if (path.endsWith('y')) {
            path = path.substring(0, path.length - 1) + 'ies';
        }
        else {
            path = path + 's';
        }
        return path;
    }
};
GenericFilterRepository.ctorParameters = () => [
    { type: HttpClient }
];
GenericFilterRepository = __decorate([
    Injectable()
], GenericFilterRepository);

let DataTableFilterComponent = class DataTableFilterComponent {
    constructor(genericRepository) {
        this.genericRepository = genericRepository;
        this.settings = new ComboboxSettings();
        this.comboboxStyle = new ComboboxStyles();
        this.optionsModel = [];
        this.triggerFilterChanged = true;
        this.onFilterChanged = new EventEmitter();
        this.comboboxItemsLoaded = new EventEmitter();
    }
    onSelectFilterChanged($event) {
        if ($event.length == 0) {
            this.filter.value = undefined;
        }
        else {
            var selectedItems = $event;
            this.filter.value = selectedItems.map(x => x.value);
        }
        if (this.triggerFilterChanged)
            this.onFilterChanged.emit({ filter: this.filter });
    }
    updateFilters() {
        this.triggerFilterChanged = false;
        if (this.inputFilter)
            this.inputFilter.updateFilters();
        if (this.dateFilter)
            this.dateFilter.updateFilters();
        if (this.booleanFilter)
            this.booleanFilter.updateFilters();
        if (this._combobox) {
            if (this.filter && this.filter.items) {
                this._combobox.selectedItems = this.filter.items.filter(x => x.selected == true);
            }
        }
        this.triggerFilterChanged = true;
    }
    get filter() {
        return this._filter;
    }
    set filter(v) {
        this._filter = v;
    }
    initializeComboboxSettings() {
        //this.comboboxStyle.buttonClasses = "btn btn-default btn-block";
        this.comboboxStyle.buttonStyles = {};
        this.settings.enableSearch = true;
        this.settings.searchMaxLimit = 25;
    }
    ngOnInit() {
        if (this.filter.allowMultipleSelections == false) {
        }
        else {
        }
        this.initializeComboboxSettings();
    }
    onSelectionChanged($event) {
        if ($event != this.filter.value) {
            this.filter.value = $event;
            this.searchInputChanged();
        }
    }
    onFilterOperationChanged($event) {
        if ($event != this.filter.operation) {
            this.filter.operation = $event;
            this.searchInputChanged();
        }
    }
    searchInputChanged() {
        this.onFilterChanged.emit({ filter: this.filter });
    }
    filterChanged($event) {
        this.onFilterChanged.emit({ filter: this.filter });
    }
    // Auto filter loader
    setFilter(items, operator = undefined) {
        if (this.filter.type == 4) {
            this.filter.items = items;
            this.filter.loading = false;
            if (operator) {
                this.filter.operation = operator;
            }
        }
    }
    createComboboxPromise(promiseLoader = new PromiseParameters()) {
        if (this.filter.autoFilterRequestEnabled == false)
            return;
        var path = this.filter.sourceTable;
        if (path === undefined || path == '')
            return;
        var filter = undefined;
        if (promiseLoader.filterKey != undefined && promiseLoader.filterKey.length > 0) {
            let criteria = new SearchCriteria();
            criteria.keys = this.filter.sourceTableSearchKeys;
            criteria.value = promiseLoader.filterKey;
            filter = [criteria];
        }
        this.comboboxPromise = this.genericRepository.getAll(this.getPath(path), promiseLoader.pageIndex, promiseLoader.pageCount, false, filter, undefined, undefined, false, true);
    }
    getPath(input) {
        var path = input.replace(/([a-z])([A-Z])/g, '$1-$2');
        path = this.pluralize(path);
        return path;
    }
    splitCamel(element) {
        var path = element.replace(/([a-z])([A-Z])/g, '$1 $2');
        return path;
    }
    pluralize(element) {
        var path = element;
        if (path.endsWith('y')) {
            path = path.substring(0, path.length - 1) + 'ies';
        }
        else {
            path = path + 's';
        }
        return path;
    }
    set combobox(combobox) {
        this._combobox = combobox;
        if (this._combobox !== undefined) {
            combobox.context = this;
            combobox.onSuccess = this.comboboxPromiseLoaded;
            this.filter.combobox = combobox;
        }
    }
    comboboxPromiseLoaded($event) {
        var items = [];
        $event.entities.forEach(entity => {
            let item = new FilterItem();
            item.key = entity["dataDisplayText"];
            item.value = entity["id"];
            items.push(item);
        });
        if (this.filter.items === undefined)
            this.filter.items = items;
        else
            this.filter.items.push(...items);
        this.comboboxItemsLoaded.emit(items);
        return items;
    }
};
DataTableFilterComponent.ctorParameters = () => [
    { type: GenericFilterRepository }
];
__decorate([
    ViewChild('inputFilter')
], DataTableFilterComponent.prototype, "inputFilter", void 0);
__decorate([
    ViewChild('dateFilter')
], DataTableFilterComponent.prototype, "dateFilter", void 0);
__decorate([
    ViewChild('booleanFilter')
], DataTableFilterComponent.prototype, "booleanFilter", void 0);
__decorate([
    Input()
], DataTableFilterComponent.prototype, "filter", null);
__decorate([
    Output()
], DataTableFilterComponent.prototype, "onFilterChanged", void 0);
__decorate([
    ViewChild("combobox")
], DataTableFilterComponent.prototype, "combobox", null);
__decorate([
    Output()
], DataTableFilterComponent.prototype, "comboboxItemsLoaded", void 0);
DataTableFilterComponent = __decorate([
    Component({
        selector: 'data-table-filter',
        template: "<input-filter #inputFilter *ngIf=\"filter.type == 0 || filter.type == 1\" [(filter)]=\"filter\" (onFilterChanged)=\"filterChanged($event)\">\r\n</input-filter>\r\n\r\n<date-filter #dateFilter *ngIf=\"filter.type == 2\" [filter]=\"filter\" (onFilterChanged)=\"filterChanged($event)\"></date-filter>\r\n\r\n<boolean-filter #booleanFilter *ngIf=\"filter.type == 3\" [filter]=\"filter\" (onFilterChanged)=\"filterChanged($event)\"></boolean-filter>\r\n\r\n<app-combobox *ngIf=\"filter.type == 4\" #combobox [showLabel]=\"false\" [styles]=\"comboboxStyle\" [(settings)]=\"settings\"\r\n    [enablePromise]=\"filter.autoFilterRequestEnabled\" [rawItems]=\"filter.items\" (selectedItemsChange)=\"onSelectFilterChanged($event)\"\r\n    [(request)]=\"comboboxPromise\" (promiseLoader)=\"createComboboxPromise($event)\">\r\n</app-combobox>",
        providers: [GenericFilterRepository]
    })
], DataTableFilterComponent);

let MultiSelectSearchFilter = class MultiSelectSearchFilter {
    constructor() {
        this._searchCache = {};
        this._searchCacheInclusive = {};
    }
    transform(options, str, limit = 0, renderLimit = 0) {
        str = (str || '').toLowerCase();
        // Drop cache because options were updated
        if (options !== this._lastOptions) {
            this._lastOptions = options;
            this._searchCache = {};
            this._searchCacheInclusive = {};
        }
        const isUnderLimit = options.length <= limit;
        if (this._searchCache[str]) {
            return isUnderLimit ? this._searchCache[str] : this._limitRenderedItems(this._searchCache[str], renderLimit);
        }
        const prevStr = str.slice(0, -1);
        const prevResults = this._searchCache[prevStr];
        if (prevResults) {
            const prevInclusiveOrIdx = this._searchCacheInclusive[prevStr];
            if (prevInclusiveOrIdx === true) {
                // If have previous results and it was inclusive, do only subsearch
                options = prevResults;
            }
            else if (typeof prevInclusiveOrIdx === 'number') {
                // Or reuse prev results with unchecked ones
                options = [...prevResults, ...options.slice(prevInclusiveOrIdx)];
            }
        }
        const optsLength = options.length;
        const maxFound = limit > 0 ? Math.min(limit, optsLength) : optsLength;
        const filteredOpts = [];
        const regexp = new RegExp(this._escapeRegExp(str), 'i');
        const matchPredicate = (option) => regexp.test(option.key), getChildren = (option) => options.filter(child => child.parentId === option.value), getParent = (option) => options.find(parent => option.parentId === parent.value);
        let i = 0, founded = 0;
        for (; i < optsLength && founded < maxFound; ++i) {
            const option = options[i];
            const directMatch = regexp.test(option.key);
            if (directMatch) {
                filteredOpts.push(option);
                founded++;
                continue;
            }
            if (typeof (option.parentId) === 'undefined') {
                const childrenMatch = getChildren(option).some(matchPredicate);
                if (childrenMatch) {
                    filteredOpts.push(option);
                    founded++;
                    continue;
                }
            }
            if (typeof (option.parentId) !== 'undefined') {
                const parentMatch = matchPredicate(getParent(option));
                if (parentMatch) {
                    filteredOpts.push(option);
                    founded++;
                    continue;
                }
            }
        }
        this._searchCache[str] = filteredOpts;
        this._searchCacheInclusive[str] = i === optsLength || i + 1;
        return isUnderLimit ? filteredOpts : this._limitRenderedItems(filteredOpts, renderLimit);
    }
    _limitRenderedItems(items, limit) {
        return items.length > limit && limit > 0 ? items.slice(0, limit) : items;
    }
    _escapeRegExp(str) {
        return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
    }
};
MultiSelectSearchFilter = __decorate([
    Pipe({
        name: 'searchFilter'
    })
], MultiSelectSearchFilter);

let BaseFilter = class BaseFilter {
    constructor() {
        this.triggerFilterChanged = true;
        this.onFilterChanged = new EventEmitter();
    }
    updateFilters() {
        this.triggerFilterChanged = false;
        this.filterValue = this.filter.value;
        this.triggerFilterChanged = true;
    }
};
__decorate([
    Input()
], BaseFilter.prototype, "filter", void 0);
__decorate([
    Output()
], BaseFilter.prototype, "onFilterChanged", void 0);
BaseFilter = __decorate([
    Directive()
], BaseFilter);

let InputFilterComponent = class InputFilterComponent extends BaseFilter {
    constructor() {
        super();
    }
    ngOnInit() {
        if (this.filter !== undefined && this.filter.value) {
            this.filter.value = this.filter.value.trim();
            this.filterValue = this.filter.value;
        }
    }
    onFilterOperationChanged($event) {
        if (this.filter.operation != $event) {
            this.filter.operation = $event;
            if (this.triggerFilterChanged)
                this.onFilterChanged.emit({ filter: this.filter });
        }
    }
    searchInputChanged() {
        if (this.filterValue != this.filter.value) {
            this.filter.value = this.filterValue;
            if (this.triggerFilterChanged)
                this.onFilterChanged.emit({ filter: this.filter });
        }
    }
};
InputFilterComponent = __decorate([
    Component({
        selector: 'input-filter',
        template: "<div class=\"input-group\">\r\n  <input class=\"form-control\" [(ngModel)]=\"filterValue\" (ngModelChange)=\"searchInputChanged()\" placeholder=\"{{filter.placeholder}}\"\r\n    style=\"min-width: 125px;\" />\r\n  <div *ngIf=\"filter.type == 0\" class=\"input-group-btn\">\r\n    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\"\r\n      aria-expanded=\"false\" style=\"box-shadow: none !important; padding-left: 8px; padding-right: 8px;\">\r\n      <i class=\"fa\" [class.fa-align-center]=\"filter.operation == undefined\" [class.fa-align-left]=\"filter.operation == 'StartsWith'\"\r\n        [class.fa-align-right]=\"filter.operation == 'EndsWith'\" [class.fa-align-justify]=\"filter.operation == 'Matches'\"></i>\r\n    </button>\r\n    <ul class=\"dropdown-menu pull-right\">\r\n      <li><a (click)=\"onFilterOperationChanged(undefined)\"><i class=\"fa fa-align-center\"></i> \u0130\u00E7inde\r\n          ge\u00E7en</a></li>\r\n      <li><a (click)=\"onFilterOperationChanged('StartsWith')\"><i class=\"fa fa-align-left\"></i>\u0130le\r\n          ba\u015Flayan</a></li>\r\n      <li><a (click)=\"onFilterOperationChanged('EndsWith')\"><i class=\"fa fa-align-right\"></i>\u0130le biten</a></li>\r\n      <li><a (click)=\"onFilterOperationChanged('Matches')\"><i class=\"fa fa-align-justify\"></i>E\u015Fle\u015Fen</a></li>\r\n    </ul>\r\n  </div><!-- /btn-group -->\r\n\r\n  <div *ngIf=\"filter.type == 1\" class=\"input-group-btn\">\r\n    <button type=\"button\" class=\"btn btn-default dropdown-toggle\" data-toggle=\"dropdown\" aria-haspopup=\"true\"\r\n      aria-expanded=\"false\" style=\"box-shadow: none !important; padding-left: 8px; padding-right: 8px;\">\r\n      <ng-template [ngIf]=\"filter.operation != undefined\">{{filter.operation}}</ng-template>\r\n      <ng-template [ngIf]=\"filter.operation == undefined\">=</ng-template>\r\n    </button>\r\n    <ul class=\"dropdown-menu dropdown-menu-right\" style=\"min-width: 50px; text-align: center;\">\r\n      <li><a (click)=\"onFilterOperationChanged(undefined)\">=</a></li>\r\n      <li><a (click)=\"onFilterOperationChanged('>')\">></a></li>\r\n      <li><a (click)=\"onFilterOperationChanged('>=')\">>=</a></li>\r\n      <li><a (click)=\"onFilterOperationChanged('<')\">\r\n          <</a> </li> <li><a (click)=\"onFilterOperationChanged('<=')\">\r\n              <=</a> </li> </ul> </div> <!-- /btn-group -->\r\n\r\n  </div><!-- /input-group -->"
    })
], InputFilterComponent);

let BooleanFilterComponent = class BooleanFilterComponent extends BaseFilter {
    constructor() {
        super();
        this.checked = undefined;
    }
    toggle() {
        if (this.filter.value === undefined)
            this.filter.value = true;
        else if (this.filter.value === true)
            this.filter.value = false;
        else if (this.filter.value === false)
            this.filter.value = undefined;
        this.filter.value = this.filter.value;
        if (this.triggerFilterChanged)
            this.onFilterChanged.emit({ filter: this.filter });
    }
    ngOnInit() {
    }
};
BooleanFilterComponent = __decorate([
    Component({
        selector: 'boolean-filter',
        template: "<div class=\"form-group data-table-noselect\" style=\"cursor:pointer; \" (click)=\"toggle()\">\r\n  <div class=\"data-table-checkbox pull-left\">\r\n    <div *ngIf=\"filter.value == true\" class=\"data-table-checkbox-selected\" style=\"position:relative\">\r\n      <i class=\"fa fa-check\" style=\"color: #5583ae;position: absolute;top: 7px;left: 5px;font-size: 20px;\"></i>\r\n    </div>\r\n    <div *ngIf=\"filter.value == false\" class=\"data-table-checkbox-selected\" style=\"position:relative\">\r\n      <i class=\"fa fa-remove\" style=\"color: #5583ae;position: absolute;top: 7px;left: 7px;font-size: 20px;\"></i>\r\n    </div>\r\n  </div>\r\n  <div class=\"clearfix\"></div>\r\n</div>",
        styles: [".data-table-greyed-out{opacity:.5;cursor:no-drop}.data-table-checkbox{width:30px;height:30px;border-radius:4px;background-color:#fff;border:1px solid #c2cad8;margin:5px 10px 5px 5px}.data-table-checkbox-selected{width:30px;height:30px;border-radius:4px}.data-table-noselect{-webkit-touch-callout:none;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}"]
    })
], BooleanFilterComponent);

const MULTISELECT_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => SelectFilterComponent),
    multi: true
};
let SelectFilterComponent = class SelectFilterComponent extends BaseFilter {
    constructor(element, fb, searchFilter, differs) {
        super();
        this.element = element;
        this.fb = fb;
        this.searchFilter = searchFilter;
        this.filterControl = this.fb.control('');
        this.disabled = false;
        this.disabledSelection = false;
        this.selectionLimitReached = new EventEmitter();
        this.dropdownClosed = new EventEmitter();
        this.dropdownOpened = new EventEmitter();
        this.onAdded = new EventEmitter();
        this.onRemoved = new EventEmitter();
        this.onLazyLoad = new EventEmitter();
        this.onFilter = this.filterControl.valueChanges;
        this.destroyed$ = new Subject();
        this.filteredOptions = [];
        this.renderFilteredOptions = [];
        this.model = [];
        this.numSelected = 0;
        this.renderItems = true;
        this.checkAllSearchRegister = new Set();
        this.checkAllStatus = false;
        this.loadedValueIds = [];
        this.defaultSettings = {
            closeOnClickOutside: true,
            enableSearch: false,
            searchRenderLimit: 0,
            searchRenderAfter: 1,
            searchMaxLimit: 0,
            searchMaxRenderedItems: 0,
            selectionLimit: 0,
            minSelectionLimit: 0,
            closeOnSelect: false,
            autoUnselect: true,
            showCheckAll: false,
            showUncheckAll: false,
            fixedTitle: false,
            dynamicTitleMaxItems: 3,
            maxHeight: '300px',
            isLazyLoad: false,
            stopScrollPropagation: false,
            loadViewDistance: 1,
            selectAddedValues: false
        };
        this.defaultTexts = {
            checkAll: 'Check all',
            uncheckAll: 'Uncheck all',
            checked: 'checked',
            checkedPlural: 'checked',
            searchPlaceholder: 'Search...',
            searchEmptyResult: 'Nothing found...',
            searchNoRenderText: 'Type in search box to see results...',
            defaultTitle: 'Select',
            allSelected: 'All selected',
        };
        this._isVisible = false;
        this._workerDocClicked = false;
        this.onModelChange = (_) => { };
        this.onModelTouched = () => { };
        this.differ = differs.find([]).create(null);
        this.settings = this.defaultSettings;
        this.texts = this.defaultTexts;
    }
    onClick(target) {
        if (!this.isVisible || !this.settings.closeOnClickOutside)
            return;
        let parentFound = false;
        while (target != null && !parentFound) {
            if (target === this.element.nativeElement) {
                parentFound = true;
            }
            target = target.parentElement;
        }
        if (!parentFound) {
            this.isVisible = false;
            this.dropdownClosed.emit();
        }
    }
    set isVisible(val) {
        this._isVisible = val;
        this._workerDocClicked = val ? false : this._workerDocClicked;
    }
    get isVisible() {
        return this._isVisible;
    }
    get searchLimit() {
        return this.settings.searchRenderLimit;
    }
    get searchRenderAfter() {
        return this.settings.searchRenderAfter;
    }
    get searchLimitApplied() {
        return this.searchLimit > 0 && this.options.length > this.searchLimit;
    }
    getItemStyle(option) {
        if (!option.isLabel) {
            return { 'cursor': 'pointer' };
        }
    }
    getItemStyleSelectionDisabled() {
        if (this.disabledSelection) {
            return { 'cursor': 'default' };
        }
    }
    ngOnInit() {
        this.settings = Object.assign(this.defaultSettings, this.settings);
        this.texts = Object.assign(this.defaultTexts, this.texts);
        this.title = this.texts.defaultTitle || '';
        this.filterControl.valueChanges
            .pipe(takeUntil(this.destroyed$))
            .subscribe(function () {
            this.updateRenderItems();
            if (this.settings.isLazyLoad) {
                this.load();
            }
        }.bind(this));
    }
    ngOnChanges(changes) {
        if (changes['options']) {
            var previousSelection = this.model;
            this.options = this.options || [];
            this.parents = this.options
                .filter(option => typeof option.parentId === 'number')
                .map(option => option.parentId);
            this.updateRenderItems();
            if (this.settings.selectAddedValues && this.loadedValueIds.length === 0) {
                this.loadedValueIds = this.loadedValueIds.concat(changes.options.currentValue.map(value => value.id));
            }
            if (this.settings.selectAddedValues && changes.options.previousValue) {
                let addedValues = changes.options.currentValue.filter(value => this.loadedValueIds.indexOf(value.id) === -1);
                this.loadedValueIds.concat(addedValues.map(value => value.id));
                if (this.checkAllStatus) {
                    this.addChecks(addedValues);
                }
                else if (this.checkAllSearchRegister.size > 0) {
                    this.checkAllSearchRegister.forEach(searchValue => this.addChecks(this.applyFilters(addedValues, searchValue)));
                }
            }
            if (this.texts) {
                this.updateTitle();
            }
            if (previousSelection != this.model) {
                this.onModelChange(this.model);
                this.onModelTouched();
            }
        }
        if (changes['texts'] && !changes['texts'].isFirstChange()) {
            this.updateTitle();
        }
    }
    ngOnDestroy() {
        this.destroyed$.next();
    }
    updateRenderItems() {
        this.renderItems = !this.searchLimitApplied || this.filterControl.value.length >= this.searchRenderAfter;
        this.filteredOptions = this.applyFilters(this.options, this.settings.isLazyLoad ? '' : this.filterControl.value);
        this.renderFilteredOptions = this.renderItems ? this.filteredOptions : [];
    }
    applyFilters(options, value) {
        return (this.searchFilter.transform(options, value, this.settings.searchMaxLimit, this.settings.searchMaxRenderedItems));
    }
    writeValue(value) {
        if (value !== undefined && value !== null) {
            this.model = Array.isArray(value) ? value : [value];
        }
        else {
            this.model = [];
        }
    }
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    ngDoCheck() {
        const changes = this.differ.diff(this.model);
        if (changes) {
            this.updateNumSelected();
            this.updateTitle();
        }
    }
    validate(_c) {
        return (this.model && this.model.length) ? null : {
            required: {
                valid: false,
            },
        };
    }
    registerOnValidatorChange(_fn) {
        throw new Error('Method not implemented.');
    }
    clearSearch(event) {
        if (event.stopPropagation) {
            event.stopPropagation();
        }
        this.filterControl.setValue('');
    }
    toggleDropdown() {
        this.isVisible = !this.isVisible;
        this.isVisible ? this.dropdownOpened.emit() : this.dropdownClosed.emit();
    }
    isSelected(option) {
        return this.model && this.model.indexOf(option.value) > -1;
    }
    setSelected(_event, option) {
        if (option.isLabel) {
            return;
        }
        if (!this.disabledSelection) {
            if (_event.stopPropagation) {
                _event.stopPropagation();
            }
            const index = this.model.indexOf(option.value);
            const isAtSelectionLimit = this.settings.selectionLimit > 0 && this.model.length >= this.settings.selectionLimit;
            if (index > -1) {
                const removeItem = (idx, id) => {
                    this.model.splice(idx, 1);
                    this.onRemoved.emit(id);
                };
                if ((this.settings.minSelectionLimit === undefined) || (this.numSelected > this.settings.minSelectionLimit)) {
                    removeItem(index, option.value);
                }
                const parentIndex = option.parentId && this.model.indexOf(option.parentId);
                if (parentIndex > -1) {
                    removeItem(parentIndex, option.parentId);
                }
                else if (this.parents.indexOf(option.value) > -1) {
                    this.options.filter(child => this.model.indexOf(child.value) > -1 && child.parentId === option.value)
                        .forEach(child => removeItem(this.model.indexOf(child.value), child.value));
                }
            }
            else if (isAtSelectionLimit && !this.settings.autoUnselect) {
                this.selectionLimitReached.emit(this.model.length);
                return;
            }
            else {
                const addItem = (id) => {
                    this.model.push(id);
                    this.onAdded.emit(id);
                };
                addItem(option.value);
                if (!isAtSelectionLimit) {
                    if (option.parentId) {
                        let children = this.options.filter(child => child.value !== option.value && child.parentId === option.parentId);
                        if (children.every(child => this.model.indexOf(child.value) > -1)) {
                            addItem(option.parentId);
                        }
                    }
                    else if (this.parents.indexOf(option.value) > -1) {
                        let children = this.options.filter(child => this.model.indexOf(child.value) < 0 && child.parentId === option.value);
                        children.forEach(child => addItem(child.value));
                    }
                }
                else {
                    const removedOption = this.model.shift();
                    this.onRemoved.emit(removedOption);
                }
            }
            if (this.settings.closeOnSelect) {
                this.toggleDropdown();
            }
            this.model = this.model.slice();
            this.onModelChange(this.model);
            this.onModelTouched();
        }
    }
    updateNumSelected() {
        this.numSelected = this.model.filter(id => this.parents.indexOf(id) < 0).length || 0;
    }
    updateTitle() {
        if (this.numSelected === 0 || this.settings.fixedTitle) {
            this.title = (this.texts) ? this.texts.defaultTitle : '';
        }
        else if (this.settings.displayAllSelectedText && this.model.length === this.options.length) {
            this.title = (this.texts) ? this.texts.allSelected : '';
        }
        else if (this.settings.dynamicTitleMaxItems && this.settings.dynamicTitleMaxItems >= this.numSelected) {
            this.title = this.options
                .filter((option) => this.model.indexOf(option.value) > -1)
                .map((option) => option.key)
                .join(', ');
        }
        else {
            this.title = this.numSelected
                + ' '
                + (this.numSelected === 1 ? this.texts.checked : this.texts.checkedPlural);
        }
    }
    searchFilterApplied() {
        return this.settings.enableSearch && this.filterControl.value && this.filterControl.value.length > 0;
    }
    addChecks(options) {
        let checkedOptions = options
            .filter(function (option) {
            if (this.model.indexOf(option.value) === -1) {
                this.onAdded.emit(option.value);
                return true;
            }
            return false;
        }.bind(this)).map((option) => option.value);
        this.model = this.model.concat(checkedOptions);
    }
    checkAll() {
        if (!this.disabledSelection) {
            this.addChecks(!this.searchFilterApplied() ? this.options : this.filteredOptions);
            if (this.settings.selectAddedValues) {
                if (this.searchFilterApplied() && !this.checkAllStatus) {
                    this.checkAllSearchRegister.add(this.filterControl.value);
                }
                else {
                    this.checkAllSearchRegister.clear();
                    this.checkAllStatus = true;
                }
                this.load();
            }
            this.onModelChange(this.model);
            this.onModelTouched();
        }
    }
    uncheckAll() {
        if (!this.disabledSelection) {
            let checkedOptions = this.model;
            let unCheckedOptions = (!this.searchFilterApplied() ? this.model
                : this.filteredOptions.map((option) => option.value));
            // set unchecked options only to the ones that were checked
            unCheckedOptions = checkedOptions.filter(item => this.model.includes(item));
            this.model = this.model.filter((id) => {
                if (((unCheckedOptions.indexOf(id) < 0) && (this.settings.minSelectionLimit === undefined)) || ((unCheckedOptions.indexOf(id) < this.settings.minSelectionLimit))) {
                    return true;
                }
                else {
                    this.onRemoved.emit(id);
                    return false;
                }
            });
            if (this.settings.selectAddedValues) {
                if (this.searchFilterApplied()) {
                    if (this.checkAllSearchRegister.has(this.filterControl.value)) {
                        this.checkAllSearchRegister.delete(this.filterControl.value);
                        this.checkAllSearchRegister.forEach(function (searchTerm) {
                            let filterOptions = this.applyFilters(this.options.filter(option => unCheckedOptions.includes(option.name)), searchTerm);
                            this.addChecks(filterOptions);
                        }.bind(this));
                    }
                }
                else {
                    this.checkAllSearchRegister.clear();
                    this.checkAllStatus = false;
                }
                this.load();
            }
            this.onModelChange(this.model);
            this.onModelTouched();
        }
    }
    preventCheckboxCheck(event, option) {
        if (this.settings.selectionLimit && !this.settings.autoUnselect &&
            this.model.length >= this.settings.selectionLimit &&
            this.model.indexOf(option.value) === -1 &&
            event.preventDefault) {
            event.preventDefault();
        }
    }
    isCheckboxDisabled() {
        return this.disabledSelection;
    }
    checkScrollPosition(ev) {
        let scrollTop = ev.target.scrollTop;
        let scrollHeight = ev.target.scrollHeight;
        let scrollElementHeight = ev.target.clientHeight;
        let roundingPixel = 1;
        let gutterPixel = 1;
        if (scrollTop >= scrollHeight - (1 + this.settings.loadViewDistance) * scrollElementHeight - roundingPixel - gutterPixel) {
            this.load();
        }
    }
    checkScrollPropagation(ev, element) {
        let scrollTop = element.scrollTop;
        let scrollHeight = element.scrollHeight;
        let scrollElementHeight = element.clientHeight;
        if ((ev.deltaY > 0 && scrollTop + scrollElementHeight >= scrollHeight) || (ev.deltaY < 0 && scrollTop <= 0)) {
            ev = ev || window.event;
            ev.preventDefault && ev.preventDefault();
            ev.returnValue = false;
        }
    }
    load() {
        this.onLazyLoad.emit({
            length: this.options.length,
            filter: this.filterControl.value,
            checkAllSearches: this.checkAllSearchRegister,
            checkAllStatus: this.checkAllStatus
        });
    }
    get tagTextInput() {
        if (this._tagTextInput) {
            this._tagTextInput.nativeElement.focus();
        }
        return this._tagTextInput;
    }
    set tagTextInput(v) {
        this._tagTextInput = v;
        if (this._tagTextInput != undefined)
            this._tagTextInput.nativeElement.focus();
    }
};
SelectFilterComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: FormBuilder },
    { type: MultiSelectSearchFilter },
    { type: IterableDiffers }
];
__decorate([
    Input()
], SelectFilterComponent.prototype, "options", void 0);
__decorate([
    Input()
], SelectFilterComponent.prototype, "settings", void 0);
__decorate([
    Input()
], SelectFilterComponent.prototype, "texts", void 0);
__decorate([
    Input()
], SelectFilterComponent.prototype, "disabled", void 0);
__decorate([
    Input()
], SelectFilterComponent.prototype, "disabledSelection", void 0);
__decorate([
    Output()
], SelectFilterComponent.prototype, "selectionLimitReached", void 0);
__decorate([
    Output()
], SelectFilterComponent.prototype, "dropdownClosed", void 0);
__decorate([
    Output()
], SelectFilterComponent.prototype, "dropdownOpened", void 0);
__decorate([
    Output()
], SelectFilterComponent.prototype, "onAdded", void 0);
__decorate([
    Output()
], SelectFilterComponent.prototype, "onRemoved", void 0);
__decorate([
    Output()
], SelectFilterComponent.prototype, "onLazyLoad", void 0);
__decorate([
    Output()
], SelectFilterComponent.prototype, "onFilter", void 0);
__decorate([
    HostListener('document: click', ['$event.target'])
], SelectFilterComponent.prototype, "onClick", null);
__decorate([
    ViewChild('tagTextInput')
], SelectFilterComponent.prototype, "tagTextInput", null);
SelectFilterComponent = __decorate([
    Component({
        selector: 'select-filter',
        template: "<ng-template [ngIf]=\"filter.loading == true\">\r\n  Filtre y\u00FCkleniyor\r\n</ng-template>\r\n\r\n<ng-template [ngIf]=\"filter.loading == false && filter.items.length > 0\">\r\n\r\n  <div class=\"dropdown\" [ngClass]=\"settings.containerClasses\" [class.open]=\"isVisible\">\r\n    <button type=\"button\" class=\"dropdown-toggle\" [ngClass]=\"settings.buttonClasses\" (click)=\"toggleDropdown()\"\r\n      [disabled]=\"disabled\">{{ title }}\r\n      <span class=\"caret\"></span>\r\n    </button>\r\n    <ul #scroller *ngIf=\"isVisible\" class=\"dropdown-menu\"\r\n      (scroll)=\"settings.isLazyLoad ? checkScrollPosition($event) : null\"\r\n      (wheel)=\"settings.stopScrollPropagation ? checkScrollPropagation($event, scroller) : null\"\r\n      [class.pull-right]=\"settings.pullRight\" [class.dropdown-menu-right]=\"settings.pullRight\"\r\n      [style.max-height]=\"settings.maxHeight\" style=\"display: block; height: auto; overflow-y: auto;\">\r\n      <li class=\"dropdown-item search\" *ngIf=\"settings.enableSearch\">\r\n        <div class=\"input-group input-group-sm\">\r\n          <span class=\"input-group-addon\" id=\"sizing-addon3\">\r\n            <i class=\"fa fa-search\"></i>\r\n          </span>\r\n          <input #tagTextInput type=\"text\" class=\"form-control\" placeholder=\"{{ texts.searchPlaceholder }}\"\r\n            aria-describedby=\"sizing-addon3\" [formControl]=\"filterControl\" autofocus>\r\n          <span class=\"input-group-btn\" *ngIf=\"filterControl.value.length > 0\">\r\n            <button class=\"btn btn-default btn-secondary\" type=\"button\" (click)=\"clearSearch($event)\">\r\n              <i class=\"fa fa-times\"></i>\r\n            </button>\r\n          </span>\r\n        </div>\r\n      </li>\r\n      <li class=\"dropdown-divider divider\" *ngIf=\"settings.enableSearch\"></li>\r\n      <li class=\"dropdown-item check-control check-control-check\" *ngIf=\"settings.showCheckAll && !disabledSelection\">\r\n        <a href=\"javascript:;\" role=\"menuitem\" tabindex=\"-1\" (click)=\"checkAll()\">\r\n          <span style=\"width: 16px;\" class=\"fa fa-check\"></span>\r\n          {{ texts.checkAll }}\r\n        </a>\r\n      </li>\r\n      <li class=\"dropdown-item check-control check-control-uncheck\"\r\n        *ngIf=\"settings.showUncheckAll && !disabledSelection\">\r\n        <a href=\"javascript:;\" role=\"menuitem\" tabindex=\"-1\" (click)=\"uncheckAll()\">\r\n          <span style=\"width: 16px;\"\r\n            [ngClass]=\"{'fa fa-trash': settings.checkedStyle !== 'fontawesome','fa fa-times': settings.checkedStyle === 'fontawesome'}\"></span>\r\n          {{ texts.uncheckAll }}\r\n        </a>\r\n      </li>\r\n      <li *ngIf=\"settings.showCheckAll || settings.showUncheckAll\" class=\"dropdown-divider divider\"></li>\r\n      <li *ngIf=\"!renderItems\" class=\"dropdown-item empty\">{{ texts.searchNoRenderText }}</li>\r\n      <li *ngIf=\"renderItems && !renderFilteredOptions.length\" class=\"dropdown-item empty\">{{ texts.searchEmptyResult\r\n        }}</li>\r\n      <li class=\"dropdown-item\" *ngFor=\"let option of renderFilteredOptions\" (click)=\"setSelected($event, option)\"\r\n        [ngStyle]=\"getItemStyle(option)\" [ngClass]=\"option.classes\" [class.dropdown-header]=\"option.isLabel\">\r\n        <a *ngIf=\"!option.isLabel; else label\" href=\"javascript:;\" role=\"menuitem\" tabindex=\"-1\"\r\n          [style.padding-left]=\"this.parents.length>0&&this.parents.indexOf(option.value)<0&&'30px'\"\r\n          [ngStyle]=\"getItemStyleSelectionDisabled()\">\r\n          <ng-container [ngSwitch]=\"settings.checkedStyle\">\r\n            <input *ngSwitchCase=\"'checkboxes'\" type=\"checkbox\" [checked]=\"isSelected(option)\"\r\n              (click)=\"preventCheckboxCheck($event, option)\" [disabled]=\"isCheckboxDisabled()\"\r\n              [ngStyle]=\"getItemStyleSelectionDisabled()\" />\r\n            <span *ngSwitchCase=\"'fa'\" style=\"width: 16px;\" class=\"fa\" [class.fa-check]=\"isSelected(option)\"></span>\r\n            <span *ngSwitchCase=\"'fontawesome'\" style=\"width: 16px;display: inline-block;\">\r\n              <i *ngIf=\"isSelected(option)\" class=\"fa fa-check\" aria-hidden=\"true\"></i>\r\n            </span>\r\n          </ng-container>\r\n          <span [ngClass]=\"settings.itemClasses\"\r\n            [style.font-weight]=\"this.parents.indexOf(option.value)>=0?'bold':'normal'\">\r\n            {{ option.key }}\r\n          </span>\r\n        </a>\r\n        <ng-template #label>{{ option.key }}</ng-template>\r\n      </li>\r\n    </ul>\r\n  </div>\r\n\r\n</ng-template>\r\n<ng-template *ngIf=\"filter.loading == false && filter.items.length == 0\">\r\n  Filtre bulunamad\u0131\r\n</ng-template>",
        providers: [MULTISELECT_VALUE_ACCESSOR, MultiSelectSearchFilter],
        styles: ["a{outline:0!important}.dropdown-inline{display:inline-block}.dropdown-toggle .caret{margin-left:4px;white-space:nowrap;display:inline-block}"]
    })
], SelectFilterComponent);

class DateHelper {
    static removeTimezoneEffect(date) {
        var utcDate = new Date(date);
        utcDate.setTime(utcDate.getTime() - (utcDate.getTimezoneOffset() * 60 * 1000));
        return utcDate;
    }
    static addTimezoneEffect(date) {
        var utcDate = new Date(date);
        utcDate.setTime(utcDate.getTime() + (utcDate.getTimezoneOffset() * 60 * 1000));
        return utcDate;
    }
}

let DateFilterComponent = class DateFilterComponent extends BaseFilter {
    constructor() {
        super();
    }
    get filterValue() {
        return this._filterValue;
    }
    set filterValue(v) {
        if (v == this._filterValue)
            return;
        if (v == undefined)
            this._filterValue = undefined;
        else if (v instanceof Date)
            this._filterValue = v;
        else
            this._filterValue = DateHelper.addTimezoneEffect(new Date(v));
    }
    ngOnInit() {
    }
    onFilterOperationChanged($event) {
        if ($event != this.filter.operation) {
            this.filter.operation = $event;
            if (this.triggerFilterChanged)
                this.onFilterChanged.emit({ filter: this.filter });
        }
    }
    searchInputChanged() {
        if (this.filterValue != this.filter.value) {
            if (this.filterValue === undefined) {
                this.filter.value = undefined;
            }
            else {
                var date = DateHelper.removeTimezoneEffect(new Date(this.filterValue));
                if (isNaN(date.getTime()))
                    this.filter.value = undefined;
                else
                    this.filter.value = date;
            }
            if (this.triggerFilterChanged)
                this.onFilterChanged.emit({ filter: this.filter });
        }
    }
};
DateFilterComponent = __decorate([
    Component({
        selector: 'date-filter',
        template: "<div class=\"input-group\" *ngIf=\"filter\">\r\n    <date-picker style=\"display: table-cell; vertical-align: top;\" [(selectedDate)]=\"filterValue\" (selectedDateChange)=\"searchInputChanged()\"\r\n        [shouldDisplayIcon]=\"false\"></date-picker>\r\n    <button type=\"button\" class=\"input-group-btn btn btn-default dropdown-toggle filter-input-btn\" data-toggle=\"dropdown\"\r\n        aria-haspopup=\"true\" aria-expanded=\"false\" style=\"box-shadow: none !important; padding-left: 8px; padding-right: 8px;\">\r\n        <ng-template [ngIf]=\"filter.operation != undefined\">{{filter.operation}}</ng-template>\r\n        <ng-template [ngIf]=\"filter.operation == undefined\">=</ng-template>\r\n    </button>\r\n    <ul class=\"dropdown-menu dropdown-menu-right\" style=\"min-width: 50px; text-align: center;\">\r\n        <li><a (click)=\"onFilterOperationChanged(undefined)\">=</a></li>\r\n        <li><a (click)=\"onFilterOperationChanged('>')\">></a></li>\r\n        <li><a (click)=\"onFilterOperationChanged('>=')\">>=</a></li>\r\n        <li><a (click)=\"onFilterOperationChanged('<')\">\r\n                <</a> </li> <li><a (click)=\"onFilterOperationChanged('<=')\">\r\n                        <=</a> </li> </ul> </div>"
    })
], DateFilterComponent);

registerLocaleData(localeTr, 'tr');
function HttpLoaderFactory(http) {
    return new TranslateHttpLoader(http);
}
function createTranslateLoader(http) {
    return new TranslateHttpLoader(http, '../assets/translations/i18n/', '.json');
}
const ɵ0 = (createTranslateLoader);
let ClDataTableModule = class ClDataTableModule {
};
ClDataTableModule = __decorate([
    NgModule({
        imports: [
            BrowserModule,
            CommonModule,
            FormsModule,
            ReactiveFormsModule,
            MatNativeDateModule,
            MatTooltipModule,
            ClCoreModule,
            ClCheckboxModule,
            ClComboboxModule,
            ClPagerModule,
            ClDatePickerModule,
            TranslateModule.forChild({
                loader: {
                    provide: TranslateLoader,
                    useFactory: ɵ0,
                    deps: [HttpClient]
                }
            })
        ],
        declarations: [
            DataTableComponent,
            DataTableFilterComponent,
            InputFilterComponent,
            BooleanFilterComponent,
            SelectFilterComponent,
            DateFilterComponent,
            MultiSelectSearchFilter
        ],
        exports: [CommonModule, DataTableComponent, DataTableFilterComponent, MultiSelectSearchFilter, TranslateModule],
        providers: [HttpClient,
            { provide: LOCALE_ID, useValue: 'tr' },
            { provide: MAT_DATE_LOCALE, useValue: 'tr' }
        ]
    })
], ClDataTableModule);

/*
 * Public API Surface of cl-angular-data-table
 */

/**
 * Generated bundle index. Do not edit.
 */

export { Action, ClDataTableModule, Column, Condition, DataTableComponent, DataTableFilterComponent, Filter, FilterItem, HttpLoaderFactory, MultiSelectSearchFilter, TableConfiguration, createTranslateLoader, ɵ0, GenericFilterRepository as ɵa, InputFilterComponent as ɵb, BaseFilter as ɵc, BooleanFilterComponent as ɵd, SelectFilterComponent as ɵe, DateFilterComponent as ɵf };
//# sourceMappingURL=cl-angular-data-table.js.map
